from __future__ import print_function

import sys
import unittest
import os
import copy
import time
import contextlib
import pyopenms
from   pyopenms.sysinfo import free_mem
import numpy as np


def show_mem(label):

    p = free_mem()
    p /= 1024.0 * 1024
    print((label+" ").ljust(50, "."), ": %8.2f MB" % p)
    sys.stdout.flush()


@contextlib.contextmanager
def MemTester(name):
        mem_at_start = free_mem()
        print("\n")
        show_mem("start test '%s' with" % name)
        yield
        missing = mem_at_start - free_mem()
        show_mem("end with")
        print("\n")
        assert missing <= 0.1* mem_at_start, "possible mem leak: %s at start, missing %s" % (mem_at_start, missing)



if True or int(os.environ.get("WITH_MEMLEAK_TESTS", 0)):

    class TestAll(unittest.TestCase):

        def setUp(self):
            self.mem_at_start = free_mem()

            print("\n")
            show_mem("AT THE BEGINNING ")
            print("\n")

            dirname = os.path.dirname(os.path.abspath(__file__))
            self.testfile = os.path.join(dirname, "../test.mzXML").encode()

        def tearDown(self):

            time.sleep(3)
            print("\n")
            show_mem("AT THE END ")
            print("\n")
            missing = self.mem_at_start - free_mem()
            assert missing <= 0.1* self.mem_at_start, "possible mem leak: %s at start, missing %s" % (self.mem_at_start, missing)

        def testAll(self):

            with MemTester("specs from experiment"):
                self.run_extractSpetraFromMSExperiment()

            with MemTester("copy experiment"):
                self.run_MSExperiment_copy()

            with MemTester("string_conversions1"):
                self.run_string_conversions1()

            with MemTester("string_conversions2"):
                self.run_string_conversions2()

            with MemTester("string_lists"):
                self.run_string_lists()

            with MemTester("list_conversions"):
                self.run_list_conversions()

            with MemTester("set_spec_peaks"):
                self.set_spec_peaks()

            with MemTester("set_spec_peaks2"):
                self.set_spec_peaks2()


            with MemTester("test io"):
                self.run_fileformats_io()


        def run_string_conversions1(self):

            basestr = 200000*b" "
            li = []
            for i in range(1000):
                if (i+1)%100 == 0:
                    show_mem("%4d runs" % i)
                dv = pyopenms.DataValue(basestr)
                dv = pyopenms.DataValue(basestr)
                li.append(dv)
            del li

        def run_string_conversions2(self):

            basestr = 200000*b" "
            li = []
            for i in range(1000):
                if (i+1)%100 == 0:
                    show_mem("%4d runs" % i)
                sf = pyopenms.SourceFile()
                sf.setNameOfFile(basestr)
                sf.setNameOfFile(basestr)
                li.append(sf)
            del li

        def run_string_lists(self):

            basestr = 10000*b" "
            li = []
            for i in range(100):
                if (i+1)%100 == 0:
                    show_mem("%4d runs" % i)
                sl = pyopenms.DataValue([basestr]*30)
                sl = pyopenms.DataValue([basestr]*30)
                li.append(sl)
                del sl
            del li

        def run_list_conversions(self):

            pc = pyopenms.Precursor()
            allpcs = 500*[pc]
            li = []
            for i in range(500):
                if (i+1)%100 == 0:
                    show_mem("%4d runs" % i)
                spec = pyopenms.MSSpectrum()
                spec.setPrecursors(allpcs)
                spec.setPrecursors(allpcs)
                li.append(spec)
                del spec
            del li

        def set_spec_peaks(self):

            data = np.zeros((10000,2), dtype=np.float32)
            li = []
            for i in range(1000):
                if (i+1)%100 == 0:
                    show_mem("%4d specs processed" % i)
                spec = pyopenms.MSSpectrum()
                spec.set_peaks((data[:,0], data[:,1]))
                spec.set_peaks((data[:,0], data[:,1]))
                spec.set_peaks((data[:,0], data[:,1]))
                li.append(spec)

            for spec in li:
                del spec
            del data

        def set_spec_peaks2(self):

            data = np.zeros((10000,2), dtype=np.float32)
            li = []
            for i in range(1000):
                if (i+1)%100 == 0:
                    show_mem("%4d specs processed" % i)
                spec = pyopenms.MSSpectrum()
                spec.set_peaks((data[:,0], data[:,1]))
                spec.set_peaks((data[:,0], data[:,1]))
                spec.set_peaks((data[:,0], data[:,1]))
                spec.set_peaks(spec.get_peaks())
                li.append(spec)

            for spec in li:
                del spec
            del data

        def run_extractSpetraFromMSExperiment(self):
            p = pyopenms.FileHandler()
            e = pyopenms.MSExperiment()
            p.loadExperiment(self.testfile, e)
            show_mem("data loaded")

            li = []
            print("please be patient :")
            for k in range(5):
                sys.stdout.flush()
                li.append([ e[i] for i in range(e.size()) ])
                li.append([ e[i] for i in range(e.size()) ])
                print((20*k+20), "%")

            print("\n")
            show_mem("spectra list generated")
            del li
            show_mem("spectra list deleted")
            del p
            del e

        def run_MSExperiment_copy(self):
            p = pyopenms.FileHandler()
            e1 = pyopenms.MSExperiment()
            p.loadExperiment(self.testfile, e1)
            show_mem("data loaded")

            specs = list(e1)
            for s in specs:
                for _ in range(10):
                    e1.addSpectrum(s)

            li = []
            print("please be patient :")
            N = 5
            for k in range(N):
                sys.stdout.flush()
                for __ in range(400):
                    e2 = copy.copy(e1)
                    li.append(e2)
                    e1 = copy.copy(e2)
                    li.append(e1)
                print(int(100.0*(k+1)/N), "%")

            print("\n")
            show_mem("experiment list generated")
            del li
            del e1
            del e2
            del p
            show_mem("experiment list deleted")

        def run_fileformats_io(self):
            p = pyopenms.FileHandler()
            e = pyopenms.MSExperiment()

            p.loadExperiment(self.testfile, e)
            show_mem("after load mzXML")

            ct = pyopenms.ChromatogramTools()
            ct.convertChromatogramsToSpectra(e)
            p.storeExperiment(self.testfile, e)
            show_mem("after store mzXML")

            p.loadExperiment(self.testfile, e)
            show_mem("after load mzXML")

            p = pyopenms.FileHandler()
            ct.convertSpectraToChromatograms(e, True, False)
            p.storeExperiment("../test.mzML".encode(), e)
            show_mem("after store mzML")
            p.loadExperiment("../test.mzML".encode(), e)
            show_mem("after load mzML")

            p = pyopenms.FileHandler()
            ct.convertChromatogramsToSpectra(e)
            p.storeExperiment("../test.mzData".encode(), e)
            show_mem("after store mzData")
            p.loadExperiment("../test.mzData".encode(), e)
            show_mem("after load mzData")

            del e
            del p
            del ct


if __name__ == "__main__":
    unittest.main()



from libcpp cimport bool
from Types cimport *
from BaseFeature cimport *
from Peak2D cimport *
from UniqueIdInterface cimport *
from FeatureMap cimport *
from BaseFeature cimport *
from FeatureHandle cimport *
from PeptideIdentification cimport *

cdef extern from "<OpenMS/KERNEL/ConsensusFeature.h>" namespace "OpenMS":

    # do not wrap BaseFeature, due to overloaded base methods
    # -> see Precursor.pxd

    cdef cppclass ConsensusFeature(UniqueIdInterface, BaseFeature):
        # wrap-inherits:
        #    UniqueIdInterface
        #    BaseFeature
        #
        # wrap-doc:
        #   A consensus feature spanning multiple LC-MS/MS experiments.
        #   -----
        #   A ConsensusFeature represents analytes that have been
        #   quantified across multiple LC-MS/MS experiments. Each analyte in a
        #   ConsensusFeature is linked to its original LC-MS/MS run through a
        #   unique identifier.
        #   -----
        #   Get access to the underlying features through getFeatureList()

        ConsensusFeature() nogil except +
        ConsensusFeature(ConsensusFeature &) nogil except +
        ConsensusFeature(UInt64, Peak2D, UInt64) nogil except +
        ConsensusFeature(UInt64, BaseFeature) nogil except +
        ConsensusFeature(UInt64, ConsensusFeature) nogil except +

        void computeConsensus()    nogil except +
        void computeMonoisotopicConsensus()    nogil except +
        void computeDechargeConsensus(FeatureMap, bool)    nogil except +

        void insert(UInt64 map_idx, Peak2D, UInt64 element_idx) nogil except +
        void insert(UInt64 map_idx, BaseFeature) nogil except +
        void insert(UInt64 map_idx, ConsensusFeature) nogil except +

        libcpp_vector[FeatureHandle] getFeatureList() nogil except +

        Size size() nogil except +

        bool operator==(ConsensusFeature) nogil except +
        bool operator!=(ConsensusFeature) nogil except +

        void addRatio(Ratio r) nogil except +
        void setRatios(libcpp_vector[Ratio] rs) nogil except +
        libcpp_vector[Ratio] getRatios() nogil except +

        void clear() nogil except +
        bool empty() nogil except +

        # # Returns the position range of the contained elements
        # DRange2 getPositionRange() nogil except +
        # # Returns the intensity range of the contained elements
        # DRange1 getIntensityRange() nogil except +

cdef extern from "<OpenMS/KERNEL/ConsensusFeature.h>" namespace "OpenMS::ConsensusFeature":

    # slim struct to feed the need for systematically storing of ratios ( @see MSQuantifications ).
    cdef cppclass Ratio:

      Ratio() nogil except +
      Ratio(Ratio rhs) nogil except +

      double ratio_value_
      String denominator_ref_
      String numerator_ref_
      libcpp_vector[String] description_



from libcpp.vector cimport vector as libcpp_vector
from libcpp.string cimport string as libcpp_string
from MRMFeature cimport *
from ReactionMonitoringTransition cimport *
from LightTargetedExperiment cimport *
from MSSpectrum cimport *
from MSChromatogram cimport *
from Peak1D cimport *
from ChromatogramPeak cimport *

cdef extern from "<OpenMS/KERNEL/MRMTransitionGroup.h>" namespace "OpenMS":

    cdef cppclass MRMTransitionGroup[SpectrumT, TransitionT]:

        # wrap-instances:
        #   MRMTransitionGroupCP := MRMTransitionGroup[MSChromatogram, ReactionMonitoringTransition]
        #   LightMRMTransitionGroupCP := MRMTransitionGroup[MSChromatogram, LightTransition]

        MRMTransitionGroup() nogil except +
        MRMTransitionGroup(MRMTransitionGroup[SpectrumT, TransitionT] &) nogil except +

        Size size() nogil except+

        String getTransitionGroupID() nogil except+
        void setTransitionGroupID(String tr_gr_id) nogil except +

        libcpp_vector[TransitionT] getTransitions() nogil except+
        libcpp_vector[TransitionT] getTransitionsMuteable() nogil except +
        void addTransition(TransitionT transition, String key) nogil except +
        TransitionT getTransition(String key)  nogil except +
        bool hasTransition(String key) nogil except +

        libcpp_vector[SpectrumT] getChromatograms() nogil except+
        void addChromatogram(SpectrumT chromatogram, String key) nogil except +
        SpectrumT getChromatogram(String key) nogil except +
        bool hasChromatogram(String key) nogil except +

        libcpp_vector[SpectrumT] getPrecursorChromatograms() nogil except+
        void addPrecursorChromatogram(SpectrumT chromatogram, String key) nogil except +
        SpectrumT getPrecursorChromatogram(String key) nogil except +
        bool hasPrecursorChromatogram(String key) nogil except +

        libcpp_vector[MRMFeature] getFeatures() nogil except+
        libcpp_vector[MRMFeature] getFeaturesMuteable() nogil except +
        void addFeature(MRMFeature feature) nogil except +
        MRMFeature getBestFeature() nogil except +

        void getLibraryIntensity(libcpp_vector[double] result) nogil except+
        MRMTransitionGroup[SpectrumT, TransitionT] subset(libcpp_vector[ libcpp_string ] tr_ids) nogil except +

        bool isInternallyConsistent() nogil except +

        bool chromatogramIdsMatch() nogil except +



from libcpp cimport bool
from libcpp.vector cimport vector as libcpp_vector
from DataValue cimport *
from String cimport *
from Types cimport *
from MetaInfoRegistry cimport *

cdef extern from "<OpenMS/METADATA/MetaInfo.h>" namespace "OpenMS":

    cdef cppclass MetaInfo:

        MetaInfo() nogil except +
        MetaInfo(MetaInfo) nogil except +

        # returns the value corresponding to a string
        DataValue getValue(String name) nogil except +
        # returns the value corresponding to an index
        DataValue getValue(UInt index) nogil except +

        # returns the value corresponding to a string, with default
        DataValue getValue(String name, DataValue default_value) nogil except +
        # returns the value corresponding to an index, with default
        DataValue getValue(UInt index, DataValue default_value) nogil except +

        # returns if this MetaInfo is set
        bool exists(String name) nogil except +
        # returns if this MetaInfo is set
        bool exists(UInt index) nogil except +

        # sets the DataValue corresponding to a name
        void setValue(String name, DataValue value) nogil except +
        #  sets the DataValue corresponding to an index
        void setValue(UInt index, DataValue value) nogil except +

        # Removes the DataValue corresponding to @p name if it exists
        void removeValue(String name) nogil except +
        # Removes the DataValue corresponding to @p index if it exists
        void removeValue(UInt index) nogil except +

        # returns a reference to the MetaInfoRegistry
        # static MetaInfoRegistry registry() nogil except +

        # fills the given vector with a list of all keys for which a value is set
        void getKeys(libcpp_vector[String] & keys) nogil except +

        # fills the given vector with a list of all keys for which a value is set
        void getKeys(libcpp_vector[unsigned int] & keys) nogil except +  # wrap-as:getKeysAsIntegers

        # returns if the MetaInfo is empty
        bool empty() nogil except +

        # removes all meta values
        void clear() nogil except +

        MetaInfoRegistry  registry() nogil except +



from libcpp cimport bool
from libcpp.vector cimport vector as libcpp_vector
from Types cimport *
from String cimport *

cdef extern from "<OpenMS/DATASTRUCTURES/Adduct.h>" namespace "OpenMS":

    cdef cppclass Adduct:
  
        Adduct() nogil except +
        Adduct(Adduct) nogil except + 
  
        Adduct(Int charge) nogil except +
        Adduct(Int charge, Int amount, double singleMass, String formula, double log_prob, double rt_shift, String label) nogil except +
  
        Int getCharge() nogil except +
  
        void setCharge(Int charge) nogil except +
  
        Int getAmount() nogil except +
        void setAmount(Int amount) nogil except +
  
        double getSingleMass() nogil except +
        void setSingleMass(double singleMass) nogil except +
  
        double getLogProb() nogil except +
        void setLogProb(double log_prob) nogil except +
  
        String getFormula() nogil except +
        void setFormula(String formula) nogil except +
  
        double getRTShift() nogil except +
        String getLabel() nogil except +



from MSExperiment  cimport *
from ChromatogramPeak cimport *
from Peak1D cimport *
from String cimport *
from ProgressLogger cimport *
from PeakFileOptions cimport *
from IMSDataConsumer cimport *

cdef extern from "<OpenMS/FORMAT/MzMLFile.h>" namespace "OpenMS":

    cdef cppclass MzMLFile(ProgressLogger):
        # wrap-inherits:
        #   ProgressLogger

        MzMLFile() nogil except +

        void load(const String& filename, MSExperiment &) nogil except+
        void store(const String& filename, MSExperiment &) nogil except+

        # COMMENT: store/load XML structure to/from a string
        void storeBuffer(String & output, MSExperiment exp) nogil except +
        void loadBuffer(const String& input, MSExperiment & exp) nogil except +

        void transform(const String&, IMSDataConsumer[Peak1D, ChromatogramPeak] *) nogil except + # wrap-ignore
        void transform(const String&, IMSDataConsumer[Peak1D, ChromatogramPeak] *,
                       bool skip_full_count, bool skip_first_pass) nogil except + # wrap-ignore

        void transform(const String&, IMSDataConsumer[Peak1D, ChromatogramPeak] *, MSExperiment& e) nogil except + # wrap-ignore
        void transform(const String&, IMSDataConsumer[Peak1D, ChromatogramPeak] *, MSExperiment& e,
                       bool skip_full_count, bool skip_first_pass) nogil except + # wrap-ignore

        PeakFileOptions getOptions() nogil except +
        void setOptions(PeakFileOptions) nogil except +

        bool isSemanticallyValid(const String & filename, StringList & errors, StringList & warnings) nogil except +




from libcpp.vector cimport vector as libcpp_vector
from libcpp cimport bool

from Param cimport *
from Feature cimport *
from FeatureMap cimport *
from ConsensusMap cimport *
from TransformationDescription cimport *
from PeptideIdentification cimport *

from MSExperiment cimport *
from Peak1D cimport *
from ChromatogramPeak cimport *

cdef extern from "<OpenMS/ANALYSIS/MAPMATCHING/MapAlignmentTransformer.h>" namespace "OpenMS":

    cdef cppclass MapAlignmentTransformer:

        MapAlignmentTransformer() nogil except +

        void transformRetentionTimes(MSExperiment&, TransformationDescription&, bool) nogil except +

        void transformRetentionTimes(FeatureMap&, TransformationDescription&, bool) nogil except +

        void transformRetentionTimes(ConsensusMap&, TransformationDescription&, bool) nogil except +

        void transformRetentionTimes(libcpp_vector[PeptideIdentification]&, TransformationDescription&, bool) nogil except +




from libcpp.vector cimport vector as libcpp_vector

from ChromatogramPeak cimport *
from ConsensusMap cimport *
from DefaultParamHandler cimport *
from Feature cimport *
from FeatureMap cimport *
from MSExperiment cimport *
from Param cimport *
from Peak1D cimport *
from PeptideIdentification cimport *
from ProgressLogger cimport *
from TransformationDescription cimport *

cdef extern from "<OpenMS/ANALYSIS/MAPMATCHING/MapAlignmentAlgorithmIdentification.h>" namespace "OpenMS":

    cdef cppclass MapAlignmentAlgorithmIdentification(DefaultParamHandler, ProgressLogger):
        # wrap-inherits:
        #    DefaultParamHandler
        #    ProgressLogger

        MapAlignmentAlgorithmIdentification() nogil except +
        
        void align(libcpp_vector[MSExperiment]&, libcpp_vector[TransformationDescription]&, int) nogil except +
        void align(libcpp_vector[FeatureMap]&, libcpp_vector[TransformationDescription]&, int) nogil except +
        void align(libcpp_vector[ConsensusMap]&, libcpp_vector[TransformationDescription]&, int) nogil except +
        # TODO nested STL
        void align(libcpp_vector[libcpp_vector[PeptideIdentification]]& ids, libcpp_vector[TransformationDescription]& trafos, int ref_index) nogil except + #wrap-ignore

        void setReference(MSExperiment&) nogil except +
        void setReference(FeatureMap&) nogil except +
        void setReference(ConsensusMap&) nogil except +
        void setReference(libcpp_vector[PeptideIdentification]&) nogil except +



from MSExperiment  cimport *
from FeatureMap cimport *
from Feature cimport *
from String cimport *
from libcpp.string cimport string as libcpp_string
from FileTypes cimport *
from Types cimport *
from PeakFileOptions cimport *

cdef extern from "<OpenMS/FORMAT/FileHandler.h>" namespace "OpenMS":

    cdef cppclass FileHandler:  # wrap=True
        FileHandler() nogil except +
        FileHandler(FileHandler) nogil except +

        bool loadExperiment(String, MSExperiment &) nogil except+
        void storeExperiment(String, MSExperiment) nogil except+
        bool loadFeatures(String, FeatureMap &) nogil except +

        PeakFileOptions  getOptions() nogil except +
        void setOptions(PeakFileOptions) nogil except +

#
# wrap static method:
#
cdef extern from "<OpenMS/FORMAT/FileHandler.h>" namespace "OpenMS::FileHandler":

    int getType(const String& filename) nogil except + # wrap-attach:FileHandler
    FileType getTypeByFileName(const String & filename) nogil except + # wrap-attach:FileHandler 
    FileType getTypeByContent(const String & filename) nogil except + # wrap-attach:FileHandler 
    String computeFileHash(const String & filename) nogil except + # wrap-attach:FileHandler 
    bool isSupported(FileType type_) nogil except + # wrap-attach:FileHandler 
    bool hasValidExtension(const String & filename, FileType type_) nogil except + # wrap-attach:FileHandler 

    # Returns the file name without the extension
    String stripExtension(String file) nogil except + # wrap-attach:FileHandler
    # Removes the current extension (if any) and adds a new one
    String swapExtension(String filename, FileType new_type) nogil except + # wrap-attach:FileHandler 



from Types cimport *
from libcpp.vector cimport vector as libcpp_vector
from MSExperiment cimport *
from MzMLFile cimport *
from MzXMLFile cimport *
from SwathMap cimport *
from StringList cimport *
from ExperimentalSettings cimport *

cdef extern from "<OpenMS/FORMAT/SwathFile.h>" namespace "OpenMS":
    
    cdef cppclass SwathFile(ProgressLogger) :
        # wrap-inherits:
        #  ProgressLogger
        SwathFile(SwathFile) nogil except + #wrap-ignore

        libcpp_vector[ SwathMap ] loadSplit(StringList file_list,
                                            String tmp,
                                            shared_ptr[ ExperimentalSettings ] exp_meta,
                                            String readoptions) nogil except +

        libcpp_vector[ SwathMap ] loadMzML(String file_,
                                           String tmp,
                                           shared_ptr[ ExperimentalSettings ] exp_meta,
                                           String readoptions) nogil except +

        libcpp_vector[ SwathMap ] loadMzXML(String file_,
                                            String tmp,
                                            shared_ptr[ ExperimentalSettings ] exp_meta, 
                                            String readoptions) nogil except +




import unittest
import os

import pyopenms

class TestSpectraFilter(unittest.TestCase):

    def setUp(self):
        dirname = os.path.dirname(os.path.abspath(__file__))
        self.filename = os.path.join(dirname, "test2.mzML").encode()
        self.exp = pyopenms.MSExperiment()
        pyopenms.MzMLFile().load(self.filename, self.exp)

    def test_map_BernNorm(self):
        thisfilter = pyopenms.BernNorm();

        old_firstspec = self.exp[0]
        thisfilter.filterPeakMap(self.exp)

        self.assertNotEqual(self.exp.size(), 0)
        self.assertNotEqual(old_firstspec, self.exp[0])

        # in most cases, a different spectrum is returned
        self.assertNotEqual(old_firstspec[10].getMZ(), self.exp[0][10].getMZ())
        self.assertNotEqual(old_firstspec[10].getIntensity(), self.exp[0][10].getIntensity())

    def test_spectrum_BernNorm(self):
        thisfilter = pyopenms.BernNorm();

        new_firstspec = self.exp[0]
        thisfilter.filterSpectrum(new_firstspec)

        self.assertNotEqual(new_firstspec.size(), 0)
        self.assertNotEqual(new_firstspec, self.exp[0])

        # in most cases, a different spectrum is returned
        self.assertNotEqual(new_firstspec[10].getMZ(), self.exp[0][10].getMZ())
        self.assertNotEqual(new_firstspec[10].getIntensity(), self.exp[0][10].getIntensity())

    def test_map_MarkerMower(self):
        thisfilter = pyopenms.MarkerMower();

        old_firstspec = self.exp[0]
        thisfilter.filterPeakMap(self.exp)

        # this deletes the spectrum ... 
        self.assertNotEqual(self.exp.size(), 0)
        self.assertNotEqual(old_firstspec, self.exp[0])

    def test_spectrum_MarkerMower(self):
        thisfilter = pyopenms.MarkerMower();

        new_firstspec = self.exp[0]
        self.assertNotEqual(new_firstspec.size(), 0)
        thisfilter.filterSpectrum(new_firstspec)

        # this deletes the spectrum ... 
        self.assertEqual(new_firstspec.size(), 0)

    def test_map_NLargest(self):
        thisfilter = pyopenms.NLargest();

        old_firstspec = self.exp[0]
        thisfilter.filterPeakMap(self.exp)

        self.assertNotEqual(self.exp.size(), 0)
        self.assertNotEqual(old_firstspec, self.exp[0])

        # in most cases, a different spectrum is returned
        self.assertNotEqual(old_firstspec[10].getMZ(), self.exp[0][10].getMZ())
        self.assertNotEqual(old_firstspec[10].getIntensity(), self.exp[0][10].getIntensity())

    def test_spectrum_NLargest(self):
        thisfilter = pyopenms.NLargest();

        new_firstspec = self.exp[0]
        thisfilter.filterSpectrum(new_firstspec)

        self.assertNotEqual(new_firstspec.size(), 0)
        self.assertNotEqual(new_firstspec, self.exp[0])

        # in most cases, a different spectrum is returned
        self.assertNotEqual(new_firstspec[10].getMZ(), self.exp[0][10].getMZ())
        self.assertNotEqual(new_firstspec[10].getIntensity(), self.exp[0][10].getIntensity())

    def test_map_Normalizer(self):
        thisfilter = pyopenms.Normalizer();

        old_firstspec = self.exp[0]
        thisfilter.filterPeakMap(self.exp)

        self.assertNotEqual(self.exp.size(), 0)
        self.assertNotEqual(old_firstspec, self.exp[0])

        self.assertEqual(old_firstspec[10].getMZ(), self.exp[0][10].getMZ())
        self.assertNotEqual(old_firstspec[10].getIntensity(), self.exp[0][10].getIntensity())

    def test_spectrum_Normalizer(self):
        thisfilter = pyopenms.Normalizer();

        new_firstspec = self.exp[0]
        thisfilter.filterSpectrum(new_firstspec)

        self.assertNotEqual(new_firstspec.size(), 0)
        self.assertNotEqual(new_firstspec, self.exp[0])

        # in most cases, a different spectrum is returned
        self.assertEqual(new_firstspec[10].getMZ(), self.exp[0][10].getMZ())
        self.assertNotEqual(new_firstspec[10].getIntensity(), self.exp[0][10].getIntensity())

    def test_map_ParentPeakMower(self):
        thisfilter = pyopenms.ParentPeakMower();
        param = thisfilter.getDefaults()
        param.setValue(b"window_size", 20.0, b"")
        param.setValue(b"set_to_zero", True, b"")
        thisfilter.setParameters(param)


        self.assertEqual(self.exp.size(), 5)
        self.assertEqual(self.exp[0].getMSLevel(), 1)

        map = []
        for spec in self.exp:
            if spec.getMSLevel() != 1:
                map.append(spec)

        self.exp.setSpectra(map)

        self.assertEqual(self.exp.size(), 3)
        self.assertEqual(self.exp[0].getMSLevel(), 2)
        self.assertEqual(self.exp[1].getMSLevel(), 2)
        self.assertEqual(self.exp[2].getMSLevel(), 2)

        old_firstspec = self.exp[0]
        old_secondspec = self.exp[1]

        thisfilter.filterPeakMap(self.exp)

        a = old_firstspec
        b = self.exp[0]
        self.assertEqual(a[174].getIntensity(), b[174].getIntensity())
        self.assertNotEqual(a[175].getIntensity(), b[175].getIntensity())
        self.assertNotEqual(a[201].getIntensity(), b[201].getIntensity())
        self.assertEqual(a[202].getIntensity(), b[202].getIntensity())

        # All peaks between 174 and 202 should be zero
        for i in range(175, 202, 1):
            self.assertEqual(b[i].getIntensity(), 0)
            self.assertEqual(a[i].getMZ(), b[i].getMZ())

    def test_spectrum_ParentPeakMower(self):
        thisfilter = pyopenms.ParentPeakMower();

        new_firstspec = self.exp[1]
        thisfilter.filterSpectrum(new_firstspec)

        self.assertNotEqual(new_firstspec.size(), 0)
        self.assertNotEqual(new_firstspec, self.exp[1])

        # TODO
        self.assertEqual(new_firstspec[10].getMZ(), self.exp[1][10].getMZ())
        self.assertEqual(new_firstspec[10].getIntensity(), self.exp[1][10].getIntensity())

    def test_map_Scaler(self):
        thisfilter = pyopenms.Scaler();

        old_firstspec = self.exp[0]
        thisfilter.filterPeakMap(self.exp)

        self.assertNotEqual(self.exp.size(), 0)
        self.assertNotEqual(old_firstspec, self.exp[0])

        # in most cases, a different spectrum is returned
        self.assertNotEqual(old_firstspec[10].getMZ(), self.exp[0][10].getMZ())
        self.assertNotEqual(old_firstspec[10].getIntensity(), self.exp[0][10].getIntensity())

    def test_spectrum_Scaler(self):
        thisfilter = pyopenms.Scaler();

        new_firstspec = self.exp[0]
        thisfilter.filterSpectrum(new_firstspec)

        self.assertNotEqual(new_firstspec.size(), 0)
        self.assertNotEqual(new_firstspec, self.exp[0])

        # in most cases, a different spectrum is returned
        self.assertNotEqual(new_firstspec[10].getMZ(), self.exp[0][10].getMZ())
        self.assertNotEqual(new_firstspec[10].getIntensity(), self.exp[0][10].getIntensity())

    def test_map_SqrtMower(self):
        thisfilter = pyopenms.SqrtMower();

        old_firstspec = self.exp[0]
        thisfilter.filterPeakMap(self.exp)

        self.assertNotEqual(self.exp.size(), 0)
        self.assertNotEqual(old_firstspec, self.exp[0])

        # in most cases, a different spectrum is returned
        self.assertEqual(old_firstspec[10].getMZ(), self.exp[0][10].getMZ())
        self.assertNotEqual(old_firstspec[10].getIntensity(), self.exp[0][10].getIntensity())

    def test_spectrum_SqrtMower(self):
        thisfilter = pyopenms.SqrtMower();

        new_firstspec = self.exp[0]
        thisfilter.filterSpectrum(new_firstspec)

        self.assertNotEqual(new_firstspec.size(), 0)
        self.assertNotEqual(new_firstspec, self.exp[0])

        # in most cases, a different spectrum is returned
        self.assertEqual(new_firstspec[10].getMZ(), self.exp[0][10].getMZ())
        self.assertNotEqual(new_firstspec[10].getIntensity(), self.exp[0][10].getIntensity())

    def test_map_ThresholdMower(self):
        thisfilter = pyopenms.ThresholdMower();

        old_firstspec = self.exp[0]
        thisfilter.filterPeakMap(self.exp)

        self.assertNotEqual(self.exp.size(), 0)
        self.assertNotEqual(old_firstspec, self.exp[0])

        # in most cases, a different spectrum is returned
        self.assertNotEqual(old_firstspec[10].getMZ(), self.exp[0][10].getMZ())
        self.assertNotEqual(old_firstspec[10].getIntensity(), self.exp[0][10].getIntensity())

    def test_spectrum_ThresholdMower(self):
        thisfilter = pyopenms.ThresholdMower();

        new_firstspec = self.exp[0]
        thisfilter.filterSpectrum(new_firstspec)

        self.assertNotEqual(new_firstspec.size(), 0)
        self.assertNotEqual(new_firstspec, self.exp[0])

        # in most cases, a different spectrum is returned
        self.assertNotEqual(new_firstspec[10].getMZ(), self.exp[0][10].getMZ())
        self.assertNotEqual(new_firstspec[10].getIntensity(), self.exp[0][10].getIntensity())

    def test_map_WindowMower(self):
        thisfilter = pyopenms.WindowMower();

        old_firstspec = self.exp[0]
        thisfilter.filterPeakMap(self.exp)

        self.assertNotEqual(self.exp.size(), 0)
        self.assertNotEqual(old_firstspec, self.exp[0])

        # in most cases, a different spectrum is returned
        self.assertNotEqual(old_firstspec[10].getMZ(), self.exp[0][10].getMZ())
        self.assertNotEqual(old_firstspec[10].getIntensity(), self.exp[0][10].getIntensity())

    def test_spectrum_WindowMower(self):
        thisfilter = pyopenms.WindowMower();

        new_firstspec = self.exp[0]
        thisfilter.filterPeakSpectrumForTopNInSlidingWindow(new_firstspec)

        self.assertNotEqual(new_firstspec.size(), 0)
        self.assertNotEqual(new_firstspec, self.exp[0])

        # in most cases, a different spectrum is returned
        self.assertNotEqual(new_firstspec[10].getMZ(), self.exp[0][10].getMZ())
        self.assertNotEqual(new_firstspec[10].getIntensity(), self.exp[0][10].getIntensity())

if __name__ == '__main__':
    unittest.main()



from Types cimport *
from DefaultParamHandler cimport *
from Matrix cimport *
from String cimport *
from Peak2D cimport *
from IsobaricChannelInformation cimport *

# typedef std::vector<IsobaricChannelInformation> IsobaricChannelList;
cdef extern from "<OpenMS/ANALYSIS/QUANTITATION/IsobaricQuantitationMethod.h>" namespace "OpenMS":
    
    cdef cppclass IsobaricQuantitationMethod(DefaultParamHandler) :
        # wrap-ignore
        # ABSTRACT class
        # no-pxd-import

        # wrap-inherits:
        #  DefaultParamHandler
        IsobaricQuantitationMethod() nogil except +
        IsobaricQuantitationMethod(IsobaricQuantitationMethod) nogil except + #wrap-ignore
        String getName() nogil except +
        libcpp_vector[IsobaricChannelInformation]  getChannelInformation() nogil except +
        Size getNumberOfChannels() nogil except +
        Matrix[ double ] getIsotopeCorrectionMatrix() nogil except +
        Size getReferenceChannel() nogil except +




from ProgressLogger cimport *
from libcpp.map cimport map as libcpp_map
from libcpp cimport bool
from Types cimport *
from FileTypes cimport *

from TargetedExperiment cimport *
from LightTargetedExperiment cimport *

cdef extern from "<OpenMS/ANALYSIS/OPENSWATH/TransitionTSVFile.h>" namespace "OpenMS":

    cdef cppclass TransitionTSVFile(ProgressLogger):
        # wrap-inherits:
        #    ProgressLogger

        TransitionTSVFile()                       nogil except +
        TransitionTSVFile(TransitionTSVFile)    nogil except + # wrap-ignore

        void convertTargetedExperimentToTSV(char * filename, TargetedExperiment& targeted_exp) nogil except +
    
        void convertTSVToTargetedExperiment(char * filename, FileType filetype, TargetedExperiment& targeted_exp) nogil except +
        void convertTSVToTargetedExperiment(char * filename, FileType filetype, LightTargetedExperiment& targeted_exp) nogil except +
    
        void validateTargetedExperiment(TargetedExperiment targeted_exp) nogil except +




from Types cimport *
from MSChromatogram cimport *
from MSSpectrum cimport *
from ExperimentalSettings cimport *

cdef extern from "<OpenMS/FORMAT/DATAACCESS/MSDataSqlConsumer.h>" namespace "OpenMS":
    
    cdef cppclass MSDataSqlConsumer:

        MSDataSqlConsumer(MSDataSqlConsumer) nogil except + #wrap-ignore
        MSDataSqlConsumer(String filename, bool clearData, int buffer_size) nogil except +

        void flush() nogil except +
        void consumeSpectrum(MSSpectrum & s) nogil except +
        void consumeChromatogram(MSChromatogram & c) nogil except +

        void setExpectedSize(Size expectedSpectra, Size expectedChromatograms) nogil except +
        void setExperimentalSettings(ExperimentalSettings & exp) nogil except +





from MSSpectrum cimport *
from Peak1D cimport *
from DefaultParamHandler cimport *

cdef extern from "<OpenMS/COMPARISON/SPECTRA/SpectrumAlignmentScore.h>" namespace "OpenMS":

    cdef cppclass SpectrumAlignmentScore(DefaultParamHandler):
        # wrap-inherits:
        #   DefaultParamHandler
        SpectrumAlignmentScore() nogil except +
        SpectrumAlignmentScore(SpectrumAlignmentScore) nogil except +

        double operator()(MSSpectrum &, MSSpectrum &) nogil except + #wrap-ignore
        double operator()(MSSpectrum &) nogil except + #wrap-ignore



from libcpp.vector cimport vector as libcpp_vector
from PeptideIdentification cimport *
from ProteinIdentification cimport *
from DefaultParamHandler cimport *
from ProgressLogger cimport *

cdef extern from "<OpenMS/ANALYSIS/ID/FalseDiscoveryRate.h>" namespace "OpenMS":

    cdef cppclass FalseDiscoveryRate(DefaultParamHandler):
        # wrap-inherits:
        #    DefaultParamHandler

        FalseDiscoveryRate()                  nogil except +
        FalseDiscoveryRate(FalseDiscoveryRate)   nogil except + #wrap-ignore

        void apply(libcpp_vector[PeptideIdentification] & forward_ids, libcpp_vector[PeptideIdentification] & reverse_ids) nogil except + 
        void apply(libcpp_vector[PeptideIdentification] & id) nogil except + 
        void apply(libcpp_vector[ProteinIdentification] & forward_ids, libcpp_vector[ProteinIdentification] & reverse_ids) nogil except + 
        void apply(libcpp_vector[ProteinIdentification] & id) nogil except + 




from Types cimport *
from libcpp cimport bool
from libcpp.map cimport map as libcpp_map
from libcpp.vector cimport vector as libcpp_vector
from DefaultParamHandler cimport *
from AASequence cimport *
from FASTAFile cimport *
from FeatureMap cimport *
from StringList cimport *

cdef extern from "<OpenMS/ANALYSIS/TARGETED/PrecursorIonSelectionPreprocessing.h>" namespace "OpenMS":
    
    cdef cppclass PrecursorIonSelectionPreprocessing(DefaultParamHandler) :
        # wrap-inherits:
        #  DefaultParamHandler
        PrecursorIonSelectionPreprocessing() nogil except +
        PrecursorIonSelectionPreprocessing(PrecursorIonSelectionPreprocessing) nogil except +

        # TODO STL map
        # libcpp_map[ String, libcpp_vector[ double ] ]  getProtMasses() nogil except +
        libcpp_vector[ double ]  getMasses(String acc) nogil except +
        # libcpp_map[ String, libcpp_vector[ double ] ]  getProteinRTMap() nogil except +
        # libcpp_map[ String, libcpp_vector[ double ] ]  getProteinPTMap() nogil except +
        # libcpp_map[ String, libcpp_vector[ String ] ]  getProteinPeptideSequenceMap() nogil except +
        void dbPreprocessing(String db_path, bool save) nogil except +
        void dbPreprocessing(String db_path, String rt_model_path, String dt_model_path, bool save) nogil except +
        void loadPreprocessing() nogil except +
        double getWeight(double mass) nogil except +
        double getRT(String prot_id, Size peptide_index) nogil except +
        double getPT(String prot_id, Size peptide_index) nogil except +
        void setFixedModifications(StringList & modifications) nogil except +
        # libcpp_map[ char, libcpp_vector[ String ] ]  getFixedModifications() nogil except +
        void setGaussianParameters(double mu, double sigma) nogil except +
        double getGaussMu() nogil except +
        double getGaussSigma() nogil except +
        double getRTProbability(String prot_id, Size peptide_index, Feature & feature) nogil except +
        double getRTProbability(double pred_rt, Feature & feature) nogil except +




from Types cimport *
from FilterFunctor cimport *

cdef extern from "<OpenMS/FILTERING/TRANSFORMERS/TICFilter.h>" namespace "OpenMS":
    
    cdef cppclass TICFilter(FilterFunctor) :
        # wrap-inherits:
        #  FilterFunctor
        TICFilter() nogil except +
        TICFilter(TICFilter) nogil except +
        double apply(MSSpectrum & ) nogil except +
        # POINTER # FilterFunctor * create() nogil except +
        String getProductName() nogil except +




from libcpp.vector cimport vector as libcpp_vector
from String cimport *

from FeatureMap cimport *
from ConsensusMap cimport *
from Feature cimport *

from libcpp cimport bool

cdef extern from "<OpenMS/FORMAT/EDTAFile.h>" namespace "OpenMS":

    cdef cppclass EDTAFile:

        EDTAFile() nogil except +
        EDTAFile(EDTAFile) nogil except +

        void store(String filename, FeatureMap & map) nogil except +
        void store(String filename, ConsensusMap & map)  nogil except +
        void load(String filename, ConsensusMap & consensus_map) nogil except +




from Types cimport *
from MSSpectrum cimport *
from String cimport *

# typedef std::map<String, std::vector<std::pair<double, double> > > MarkerIonsType;

ctypedef libcpp_map[String, libcpp_vector[ libcpp_pair[double, double] ] ] MarkerIonsType

cdef extern from "<OpenMS/ANALYSIS/RNPXL/RNPxlMarkerIonExtractor.h>" namespace "OpenMS":
    
    cdef cppclass RNPxlMarkerIonExtractor "OpenMS::RNPxlMarkerIonExtractor":
        RNPxlMarkerIonExtractor() nogil except + 
        RNPxlMarkerIonExtractor(RNPxlMarkerIonExtractor) nogil except + #wrap-ignore
        # MarkerIonsType extractMarkerIons(MSSpectrum & s, double marker_tolerance) nogil except +
        libcpp_map[String, libcpp_vector[ libcpp_pair[double, double] ] ] extractMarkerIons(MSSpectrum & s,
                                                                                            double marker_tolerance) nogil except +






        ## # TODO nested STL
        ## #  void computeHighestPeptides(libcpp_vector[libcpp_vector[double] ] & peptide_site_scores, libcpp_vector[ProbablePhosphoSites] & sites, libcpp_vector[libcpp_vector[Size] ] & permutations) nogil except +

        ## #Computes the site determing_ions for the given AS and sequences in candidates
        ## void compute_site_determining_ions(libcpp_vector[MSSpectrum[RichPeak1D]] & th_spectra, ProbablePhosphoSites & candidates, Int charge, libcpp_vector[MSSpectrum[RichPeak1D]] & site_determining_ions) nogil except +

        ## libcpp_vector[ size_t ] computeTupel_(AASequence & without_phospho)
        ## # TODO nested STL
        ## # libcpp_vector[ libcpp_vector[ size_t ] ] computePermutations_(libcpp_vector[ size_t ] tupel, Int number_of_phospho_sites)



from Types cimport *
from libcpp cimport bool

cdef extern from "<OpenMS/METADATA/Product.h>" namespace "OpenMS":

    cdef cppclass Product:


        Product()    nogil except +
        Product(Product)    nogil except +

        bool operator==(Product) nogil except +
        bool operator!=(Product) nogil except +

        double getMZ() nogil except +
        void setMZ(double) nogil except +

        double getIsolationWindowLowerOffset() nogil except +
        void  setIsolationWindowLowerOffset(double bound) nogil except +

        double getIsolationWindowUpperOffset() nogil except +
        void  setIsolationWindowUpperOffset(double bound) nogil except +




import sys
import pyopenms

def getTransitionGroup(exp, targeted, key, trgr_ids, chrom_map):
    r = pyopenms.LightMRMTransitionGroupCP()
    r.setTransitionGroupID(key)
    for tr in trgr_ids:
        transition = targeted.getTransitions()[tr]
        chrom_idx = chrom_map[ transition.getNativeID() ]
        chrom = exp.getChromatograms()[ chrom_idx ]
        chrom.setMetaValue("product_mz", transition.getProductMZ() )
        chrom.setMetaValue("precursor_mz", transition.getPrecursorMZ() )
        r.addTransition( transition, transition.getNativeID() )
        r.addChromatogram( chrom, chrom.getNativeID() )
    return r


def algorithm(exp, targeted, picker, scorer, trafo):

    output = pyopenms.FeatureMap()

    scorer.prepareProteinPeptideMaps_(targeted)

    chrom_map = {}
    pepmap = {}
    trmap = {}
    for i, chrom in enumerate(exp.getChromatograms()):
        chrom_map[ chrom.getNativeID() ] = i
    for i, pep in enumerate(targeted.getCompounds() ):
        pepmap[ pep.id ] = i
    for i, tr in enumerate(targeted.getTransitions() ):
        tmp = trmap.get( tr.getPeptideRef() , [])
        tmp.append( i )
        trmap[ tr.getPeptideRef() ] = tmp

    swath_maps_dummy = []
    for key, value in trmap.iteritems():
        try:
            transition_group = getTransitionGroup(exp, targeted, key, value, chrom_map)
        except Exception:
            print "Skip ", key, value
            continue
        picker.pickTransitionGroup(transition_group);
        scorer.scorePeakgroups(transition_group, trafo, swath_maps_dummy, output, False);

    return output

def main(options):
    out = options.outfile
    chromat_in = options.infile
    traml_in = options.traml_in
    trafo_in = options.trafo_in

    pp = pyopenms.MRMTransitionGroupPicker()


    metabolomics = False
    # this is an important weight for RT-deviation -- the larger the value, the less importance will be given to exact RT matches
    # for proteomics data it tends to be a good idea to set it to the length of
    # the RT space (e.g. for 100 second RT space, set it to 100)
    rt_normalization_factor = 100.0

    pp_params = pp.getDefaults();
    pp_params.setValue("PeakPickerMRM:remove_overlapping_peaks", options.remove_overlapping_peaks, '')
    pp_params.setValue("PeakPickerMRM:method", options.method, '')
    if (metabolomics):
        # Need to change those for metabolomics and very short peaks!
        pp_params.setValue("PeakPickerMRM:signal_to_noise", 0.01, '')
        pp_params.setValue("PeakPickerMRM:peak_width", 0.1, '')
        pp_params.setValue("PeakPickerMRM:gauss_width", 0.1, '')
        pp_params.setValue("resample_boundary", 0.05, '')
        pp_params.setValue("compute_peak_quality", "true", '')
    pp.setParameters(pp_params)

    scorer = pyopenms.MRMFeatureFinderScoring()
    scoring_params = scorer.getDefaults();
    # Only report the top 5 features
    scoring_params.setValue("stop_report_after_feature", 5, '')
    scoring_params.setValue("rt_normalization_factor", rt_normalization_factor, '')
    scorer.setParameters(scoring_params);

    chromatograms = pyopenms.MSExperiment()
    fh = pyopenms.FileHandler()
    fh.loadExperiment(chromat_in, chromatograms)
    targeted = pyopenms.TargetedExperiment();
    tramlfile = pyopenms.TraMLFile();
    tramlfile.load(traml_in, targeted);

    trafoxml = pyopenms.TransformationXMLFile()
    trafo = pyopenms.TransformationDescription()
    if trafo_in is not None:
        model_params = pyopenms.Param()
        model_params.setValue("symmetric_regression", "false", "", [])
        model_type = "linear"
        trafoxml.load(trafo_in, trafo, True)
        trafo.fitModel(model_type, model_params);


    light_targeted = pyopenms.LightTargetedExperiment();
    pyopenms.OpenSwathDataAccessHelper().convertTargetedExp(targeted, light_targeted)
    output = algorithm(chromatograms, light_targeted, pp, scorer, trafo)

    pyopenms.FeatureXMLFile().store(out, output);

def handle_args():
    import argparse

    usage = ""
    usage += "\nMRMTransitionGroupScorer picks transition groups in measured chromatograms (mzML) and scores them"

    parser = argparse.ArgumentParser(description = usage )
    parser.add_argument('--in', dest="infile", help = 'An input file containing chromatograms')
    parser.add_argument("--tr", dest="traml_in", help="TraML input file containt the transitions")
    parser.add_argument("--trafo", dest="trafo_in", help="Trafo input file")
    parser.add_argument("--out", dest="outfile", help="Output file with annotated chromatograms")
    parser.add_argument("--remove_overlapping_peaks", dest="remove_overlapping_peaks", default="false", help="true/false", metavar='0.1', type=str)
    parser.add_argument("--method", dest="method", default="legacy", help="legacy/corrected", metavar='0.1', type=str)

    args = parser.parse_args(sys.argv[1:])
    return args

if __name__ == '__main__':
    options = handle_args()
    main(options)




from libcpp.vector cimport vector as libcpp_vector
from DefaultParamHandler cimport *
from MSSpectrum cimport *
from MSExperiment cimport *
from ChromatogramPeak cimport *
from Peak1D cimport *

cdef extern from "<OpenMS/ANALYSIS/DENOVO/DeNovoIonScoring.h>" namespace "OpenMS":
    
    cdef cppclass DeNovoIonScoring "OpenMS::DeNovoIonScoring":
        # wrap-ignore
        # ABSTRACT CLASS
        # no-pxd-import
        DeNovoIonScoring() nogil except +
        DeNovoIonScoring(DeNovoIonScoring) nogil except +
        void getIonScores(libcpp_vector[ DeNovoIonScore ] &ion_scores, MSSpectrum &spec) nogil except +
        void getIonScores(libcpp_vector[ libcpp_vector[ DeNovoIonScore ] ] &ion_scores, MSExperiment &exp) nogil except + #wrap-ignore

cdef extern from "<OpenMS/ANALYSIS/DENOVO/DeNovoIonScoring.h>" namespace "OpenMS::DeNovoIonScoring":
    
    cdef cppclass DeNovoIonScore "OpenMS::DeNovoIonScoring::IonScore":
        DeNovoIonScore() nogil except +
        DeNovoIonScore(DeNovoIonScore) nogil except +

        # score
        double score

        # position of the ion
        double position

        # index of peak in the spectrum, -1 if not in spectrum
        ptrdiff_t index



from Types cimport *
from String cimport *
from OpenSwathDataStructures cimport *
from SpectrumAccessOpenMS cimport *
from SpectrumAccessOpenMSCached cimport *
from SpectrumAccessOpenMSInMemory cimport *
from SpectrumAccessQuadMZTransforming cimport *
from ISpectrumAccess cimport *
from ProgressLogger cimport *

# typedef boost::shared_ptr<ISpectrumAccess> SpectrumAccessPtr;

cdef extern from "<OpenMS/ANALYSIS/OPENSWATH/ChromatogramExtractorAlgorithm.h>" namespace "OpenMS":

    cdef cppclass ChromatogramExtractorAlgorithm(ProgressLogger):
        # wrap-inherits:
        #    ProgressLogger

        ChromatogramExtractorAlgorithm() nogil except +
        ChromatogramExtractorAlgorithm(ChromatogramExtractorAlgorithm) nogil except +

        # abstract base class ISpectrumAccess given as first input arg
        void extractChromatograms(
            shared_ptr[ SpectrumAccessOpenMS ] input,
            libcpp_vector[ shared_ptr[OSChromatogram] ] & output,
            libcpp_vector[ ExtractionCoordinates ] extraction_coordinates,
            double mz_extraction_window,
            bool ppm,
            double im_extraction_window,
            String filter) nogil except +

        void extractChromatograms(
            shared_ptr[ SpectrumAccessOpenMSCached ] input,
            libcpp_vector[ shared_ptr[OSChromatogram] ] & output,
            libcpp_vector[ ExtractionCoordinates ] extraction_coordinates,
            double mz_extraction_window,
            bool ppm,
            double im_extraction_window,
            String filter) nogil except +

        void extractChromatograms(
            shared_ptr[ SpectrumAccessOpenMSInMemory ] input,
            libcpp_vector[ shared_ptr[OSChromatogram] ] & output,
            libcpp_vector[ ExtractionCoordinates ] extraction_coordinates,
            double mz_extraction_window,
            bool ppm,
            double im_extraction_window,
            String filter) nogil except +

        void extractChromatograms(
            shared_ptr[ SpectrumAccessQuadMZTransforming ] input,
            libcpp_vector[ shared_ptr[OSChromatogram] ] & output,
            libcpp_vector[ ExtractionCoordinates ] extraction_coordinates,
            double mz_extraction_window,
            bool ppm,
            double im_extraction_window,
            String filter) nogil except +


        # void extract_value_tophat # -> uses iterators

cdef extern from "<OpenMS/ANALYSIS/OPENSWATH/ChromatogramExtractorAlgorithm.h>" namespace "OpenMS::ChromatogramExtractorAlgorithm":

    cdef cppclass ExtractionCoordinates:

        ExtractionCoordinates() nogil except +
        ExtractionCoordinates(ExtractionCoordinates) nogil except +

        double mz # mz around which should be extracted
        double mz_precursor # precursor m/z value (is currently ignored by the algorithm)
        double rt_start # rt start of extraction (in seconds)
        double rt_end # rt end of extraction (in seconds)
        double ion_mobility # ion mobility value around which should be extracted
        libcpp_string id # identifier




from Types cimport *
from Feature cimport *
from Peak2D cimport *
from UniqueIdInterface cimport *

cdef extern from "<OpenMS/KERNEL/FeatureHandle.h>" namespace "OpenMS":
    
    cdef cppclass FeatureHandle(Peak2D,UniqueIdInterface) :
        # wrap-inherits:
        #  Peak2D
        #  UniqueIdInterface

        FeatureHandle() nogil except +
        FeatureHandle(FeatureHandle) nogil except +

        FeatureHandle(UInt64 map_index, Peak2D & point, UInt64 element_index) nogil except +
        # FeatureHandle(UInt64 map_index, BaseFeature & feature) nogil except +
        # FeatureHandleMutable_  asMutable() nogil except +

        UInt64 getMapIndex() nogil except +
        void setMapIndex(UInt64 i) nogil except +
        void setCharge(Int charge) nogil except +
        Int getCharge() nogil except +
        void setWidth(float width) nogil except +
        float getWidth() nogil except +

        bool operator==(FeatureHandle & i) nogil except +
        bool operator!=(FeatureHandle & i) nogil except +






    # NOTE: using shared_ptr for a singleton will lead to segfaults, use raw ptr instead
    # cdef AutowrapPtrHolder[_ProteaseDB] inst

    def __init__(self):
      self.inst = AutowrapPtrHolder[_ProteaseDB](_getInstance_ProteaseDB())



from Types cimport *
from libcpp cimport bool
from IMSElement cimport *
from String cimport *
# from IMSAlphabetParser cimport *

cdef extern from "<OpenMS/CHEMISTRY/MASSDECOMPOSITION/IMS/IMSAlphabet.h>" namespace "OpenMS::ims::IMSAlphabet":

    ctypedef IMSElement element_type
    ctypedef libcpp_vector[element_type] container
    ctypedef libcpp_vector[mass_type] masses_type

cdef extern from "<OpenMS/CHEMISTRY/MASSDECOMPOSITION/IMS/IMSAlphabet.h>" namespace "OpenMS::ims":
    
    cdef cppclass IMSAlphabet "OpenMS::ims::IMSAlphabet":
        IMSAlphabet() nogil except +
        IMSAlphabet(IMSAlphabet) nogil except +

        element_type  getElement(name_type & name) nogil except +
        name_type  getName(size_type index) nogil except +
        mass_type getMass(name_type & name) nogil except +
        mass_type getMass(size_type index) nogil except +
        masses_type getMasses(size_type isotope_index) nogil except +
        masses_type getAverageMasses() nogil except +
        bool hasName(name_type & name) nogil except +
        void push_back(name_type & name, mass_type value) nogil except +
        void push_back(element_type & element) nogil except +
        void clear() nogil except +
        void sortByNames() nogil except +
        void sortByValues() nogil except +
        void load(String & fname) nogil except +
        # POINTER # void load(libcpp_string & fname, IMSAlphabetParser[] * parser) nogil except +
        IMSAlphabet(libcpp_vector[IMSElement] & elements) nogil except +
        size_type size() nogil except +
        element_type  getElement(size_type index) nogil except +
        void setElement(name_type & name, mass_type mass, bool forced) nogil except +
        bool erase(name_type & name) nogil except +




import numpy as np
from time import time
import sys

def main(argv):
    if len(argv) != 1:
        print 'Usage: numpy_benchmark.py <LOG_N>'
        return;

    logN = int(argv[0])
    N = 2**logN

    x=np.arange(N)*(1+1j)
    t1=time()
    y=np.fft.fftn(x)
    t2=time()

    print N, t2-t1

if __name__ == '__main__':
    main(sys.argv[1:])




from Types cimport *
from DefaultParamHandler cimport *
from MSExperiment cimport *
from TargetedExperiment cimport *

cdef extern from "<OpenMS/ANALYSIS/TARGETED/MRMMapping.h>" namespace "OpenMS":
    
    cdef cppclass MRMMapping(DefaultParamHandler) :
        # wrap-inherits:
        #  DefaultParamHandler
        MRMMapping() nogil except +
        MRMMapping(MRMMapping) nogil except + #wrap-ignore

        void mapExperiment(MSExperiment input_chromatograms, TargetedExperiment targeted_exp, MSExperiment& output) nogil except +




from MSSpectrum cimport *
from MSChromatogram cimport *
from ExperimentalSettings cimport *
from DataProcessing cimport *
from PeakFileOptions cimport *

cdef extern from "<OpenMS/FORMAT/DATAACCESS/MSDataWritingConsumer.h>" namespace "OpenMS":

    cdef cppclass PlainMSDataWritingConsumer:

        PlainMSDataWritingConsumer(String filename) nogil except +
        PlainMSDataWritingConsumer(PlainMSDataWritingConsumer) nogil except + #wrap-ignore

        void consumeSpectrum(MSSpectrum & s) nogil except +
        void consumeChromatogram(MSChromatogram & c) nogil except +

        void setExperimentalSettings(ExperimentalSettings& exp) nogil except +
        void setExpectedSize(Size expectedSpectra, Size expectedChromatograms) nogil except +

        void addDataProcessing(DataProcessing d) nogil except +
        Size getNrSpectraWritten()  nogil except +
        Size getNrChromatogramsWritten() nogil except +

        void setOptions(PeakFileOptions opt) nogil except +
        PeakFileOptions getOptions() nogil except +

    cdef cppclass NoopMSDataWritingConsumer:

        NoopMSDataWritingConsumer(String filename) nogil except +
        NoopMSDataWritingConsumer(NoopMSDataWritingConsumer) nogil except + #wrap-ignore

        void consumeSpectrum(MSSpectrum & s) nogil except +
        void consumeChromatogram(MSChromatogram & c) nogil except +

        void setExperimentalSettings(ExperimentalSettings& exp) nogil except +
        void setExpectedSize(Size expectedSpectra, Size expectedChromatograms) nogil except +

        void addDataProcessing(DataProcessing d) nogil except +
        Size getNrSpectraWritten()  nogil except +
        Size getNrChromatogramsWritten() nogil except +




from libcpp cimport bool
from libcpp.map cimport map as libcpp_map
from libcpp.vector cimport vector as libcpp_vector
from String cimport *
from Types cimport *

cdef extern from "<OpenMS/FORMAT/SequestInfile.h>" namespace "OpenMS":
    
    cdef cppclass SequestInfile "OpenMS::SequestInfile":
        SequestInfile() nogil except +
        SequestInfile(SequestInfile) nogil except +
        bool operator==(SequestInfile &sequest_infile) nogil except +
        void store(const String &filename) nogil except +
        String getEnzymeInfoAsString() nogil except +
        String  getDatabase() nogil except +
        void setDatabase(const String &database) nogil except +
        String  getNeutralLossesForIons() nogil except +
        void setNeutralLossesForIons(const String &neutral_losses_for_ions) nogil except +
        String  getIonSeriesWeights() nogil except +
        void setIonSeriesWeights(const String &ion_series_weights) nogil except +
        String  getPartialSequence() nogil except +
        void setPartialSequence(const String &partial_sequence) nogil except +
        String  getSequenceHeaderFilter() nogil except +
        void setSequenceHeaderFilter(const String &sequence_header_filter) nogil except +
        String  getProteinMassFilter() nogil except +
        void setProteinMassFilter(const String &protein_mass_filter) nogil except +
        float getPeakMassTolerance() nogil except +
        void setPeakMassTolerance(float peak_mass_tolerance) nogil except +
        float getPrecursorMassTolerance() nogil except +
        void setPrecursorMassTolerance(float precursor_mass_tolerance) nogil except +
        float getMatchPeakTolerance() nogil except +
        void setMatchPeakTolerance(float match_peak_tolerance) nogil except +
        float getIonCutoffPercentage() nogil except +
        void setIonCutoffPercentage(float ion_cutoff_percentage) nogil except +
        Size getPeptideMassUnit() nogil except +
        void setPeptideMassUnit(Size peptide_mass_unit) nogil except +
        Size getOutputLines() nogil except +
        void setOutputLines(Size output_lines) nogil except +
        Size getEnzymeNumber() nogil except +
        String getEnzymeName() nogil except +
        Size setEnzyme(String enzyme_name) nogil except +
        Size getMaxAAPerModPerPeptide() nogil except +
        void setMaxAAPerModPerPeptide(Size max_aa_per_mod_per_peptide) nogil except +
        Size getMaxModsPerPeptide() nogil except +
        void setMaxModsPerPeptide(Size max_mods_per_peptide) nogil except +
        Size getNucleotideReadingFrame() nogil except +
        void setNucleotideReadingFrame(Size nucleotide_reading_frame) nogil except +
        Size getMaxInternalCleavageSites() nogil except +
        void setMaxInternalCleavageSites(Size max_internal_cleavage_sites) nogil except +
        Size getMatchPeakCount() nogil except +
        void setMatchPeakCount(Size match_peak_count) nogil except +
        Size getMatchPeakAllowedError() nogil except +
        void setMatchPeakAllowedError(Size match_peak_allowed_error) nogil except +
        bool getShowFragmentIons() nogil except +
        void setShowFragmentIons(bool show_fragments) nogil except +
        bool getPrintDuplicateReferences() nogil except +
        void setPrintDuplicateReferences(bool print_duplicate_references) nogil except +
        bool getRemovePrecursorNearPeaks() nogil except +
        void setRemovePrecursorNearPeaks(bool remove_precursor_near_peaks) nogil except +
        bool getMassTypeParent() nogil except +
        void setMassTypeParent(bool mass_type_parent) nogil except +
        bool getMassTypeFragment() nogil except +
        void setMassTypeFragment(bool mass_type_fragment) nogil except +
        bool getNormalizeXcorr() nogil except +
        void setNormalizeXcorr(bool normalize_xcorr) nogil except +
        bool getResiduesInUpperCase() nogil except +
        void setResiduesInUpperCase(bool residues_in_upper_case) nogil except +
        void addEnzymeInfo(libcpp_vector[ String ] &enzyme_info) nogil except +

        libcpp_map[ String, libcpp_vector[ String ] ]  getModifications() nogil except + # wrap-ignore
        void handlePTMs(const String &modification_line, const String &modifications_filename, bool monoisotopic) nogil except +




from libcpp cimport bool
from libcpp.vector cimport vector as libcpp_vector
from SimTypes cimport *
from SvmTheoreticalSpectrumGenerator cimport *

cdef extern from "<OpenMS/CHEMISTRY/SvmTheoreticalSpectrumGeneratorSet.h>" namespace "OpenMS":
    
    cdef cppclass SvmTheoreticalSpectrumGeneratorSet "OpenMS::SvmTheoreticalSpectrumGeneratorSet":
        SvmTheoreticalSpectrumGeneratorSet() nogil except +
        SvmTheoreticalSpectrumGeneratorSet(SvmTheoreticalSpectrumGeneratorSet) nogil except +
        # void simulate(MSSpectrum &spectrum, AASequence &peptide, boost::random::mt19937_64& rng, Size precursor_charge) nogil except +
        void load(String) nogil except +
        void getSupportedCharges(libcpp_set[ size_t ] &charges) nogil except +
        SvmTheoreticalSpectrumGenerator  getSvmModel(Size) nogil except +




from MSSpectrum cimport *
from MSChromatogram cimport *
from IMSDataConsumer cimport *
from ExperimentalSettings cimport *

# see ../extra_includes/python_ms_data_consumer.hpp for actual wrapped C++ code
cdef extern from "python_ms_data_consumer.hpp":

    cdef cppclass PythonMSDataConsumer(IMSDataConsumer[Peak1D, ChromatogramPeak]):
        # wrap-ignore
        # no-pxd-import

        PythonMSDataConsumer(object py_consumer,
                             object (*spectrum_wrapper)(const MSSpectrum &),
                             object (*chromatogram_wrapper)(const MSChromatogram &),
                             object (*experimental_settings_wrapper)(const ExperimentalSettings &)
                            )

        void consumeSpectrum(MSSpectrum &) except +

        void consumeChromatogram(MSChromatogram &) except +

        void setExpectedSize(Size ns, Size, nc) except +

        void setExperimentalSettings(ExperimentalSettings &) except +















from Peak1D cimport *
from libcpp.vector cimport vector as libcpp_vector

# this class has addons, see the ./addons folder

cdef extern from "<OpenMS/FORMAT/PeakTypeEstimator.h>" namespace "OpenMS":

    cdef cppclass PeakTypeEstimator:
        PeakTypeEstimator() nogil except +

        # wrpped in ../addons/PeakTypeEstimator.pyx:
        int estimateType(libcpp_vector[Peak1D].iterator,
                         libcpp_vector[Peak1D].iterator) nogil except + # wrap-ignore



#!/usr/bin/env python
# -*- coding: utf-8  -*-
# --------------------------------------------------------------------------
#                   OpenMS -- Open-Source Mass Spectrometry
# --------------------------------------------------------------------------
# Copyright The OpenMS Team -- Eberhard Karls University Tuebingen,
# ETH Zurich, and Freie Universitaet Berlin 2002-2020.

# This software is released under a three-clause BSD license:
#  * Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#  * Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#  * Neither the name of any author or any participating institution
#    may be used to endorse or promote products derived from this software
#    without specific prior written permission.
# For a full list of authors, refer to the file AUTHORS.
# --------------------------------------------------------------------------
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL ANY OF THE AUTHORS OR THE CONTRIBUTING
# INSTITUTIONS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
# OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
# OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
# ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
# --------------------------------------------------------------------------
# $Maintainer: Hendrik Weisser $
# $Authors: Hendrik Weisser $
# --------------------------------------------------------------------------

"""Convert featureXML files between formats used before and after OpenMS 1.8"""

import sys
import re
import os.path

if len(sys.argv) < 3:
    sys.exit("Error: two parameters (input featureXML, output featureXML) "
             "expected!")

in_path = sys.argv[1]
out_path = sys.argv[2]

if os.path.realpath(in_path) == os.path.realpath(out_path):
    sys.exit("Error: output path must be different from input path!")

old_pattern = re.compile('<hullpoint>\n<hposition dim="0">(.*)</hposition>\n'
                         '<hposition dim="1">(.*)</hposition>\n</hullpoint>')
new_pattern = re.compile('<pt x="(.*)" y="(.*)" */>')

with open(in_path) as source:
    with open(out_path, "w") as sink:
        line_count = 0
        for line in source:
            line_count += 1
            stripped = line.strip()
            if stripped == "<hullpoint>": # convert old to new:
                temp_count = line_count
                indent = line[:line.index("<")]
                content = stripped
                while stripped != "</hullpoint>":
                    stripped = source.next().strip()
                    line_count += 1
                    content += "\n" + stripped
                match = old_pattern.match(content)
                if not match:
                    print "skipping unexpected content in lines %d-%d" % \
                        (temp_count, line_count)
                else:
                    sink.write(indent + '<pt x="%s" y="%s" />\n' %
                               (match.group(1), match.group(2)))
                continue
            # else:
            match = new_pattern.match(stripped)
            if match: # convert new to old:
                indent = line[:line.index("<")]
                sink.write(indent + "<hullpoint>\n")
                sink.write(indent + '\t<hposition dim="0">%s</hposition>\n' %
                           match.group(1))
                sink.write(indent + '\t<hposition dim="1">%s</hposition>\n' %
                           match.group(2))
                sink.write(indent + "</hullpoint>\n")
            else:
                sink.write(line)



from libcpp cimport bool
from libcpp.vector cimport vector as libcpp_vector
from libcpp.map cimport map as libcpp_map
from ConsensusMap cimport *
from FeatureMap cimport *
from MSExperiment cimport *

cdef extern from "<OpenMS/KERNEL/ConversionHelper.h>" namespace "OpenMS":

    cdef cppclass MapConversion:

      void convert(UInt64 input_map_index,
                   FeatureMap input_map,
                   ConsensusMap & output_map,
                   Size n) nogil except +
    
      void convert(UInt64 input_map_index,
                   MSExperiment & input_map,
                   ConsensusMap & output_map,
                   Size n) nogil except +
    
      void convert(ConsensusMap input_map,
                   bool keep_uids,
                   FeatureMap & output_map) nogil except +




from libcpp cimport bool
from Types cimport *
from String cimport *
from ResidueModification cimport *

cdef extern from "<OpenMS/CHEMISTRY/ModificationDefinition.h>" namespace "OpenMS":
    
    cdef cppclass ModificationDefinition "OpenMS::ModificationDefinition":
        # wrap-hash:
        #   getModificationName().c_str()

        ModificationDefinition() nogil except +
        ModificationDefinition(ModificationDefinition) nogil except +
        ModificationDefinition(const String &mod) nogil except +
        ModificationDefinition(const String &mod, bool fixed) nogil except +
        ModificationDefinition(const String &mod, bool fixed, UInt max_occur) nogil except +
        ModificationDefinition(ResidueModification &mod) nogil except +
        ModificationDefinition(ResidueModification &mod, bool fixed) nogil except +
        ModificationDefinition(ResidueModification &mod, bool fixed, UInt max_occur) nogil except +

        bool operator==(ModificationDefinition &rhs) nogil except +
        bool operator!=(ModificationDefinition &rhs) nogil except +
        bool operator<(ModificationDefinition &) nogil except +

        void setFixedModification(bool fixed) nogil except +
        bool isFixedModification() nogil except +
        void setMaxOccurrences(UInt num) nogil except +
        UInt getMaxOccurrences() nogil except +
        String getModificationName() nogil except +
        void setModification(const String &modification) nogil except +

        ResidueModification getModification() nogil except +




#!/usr/bin/env python
# -*- coding: utf-8  -*-
"""
--------------------------------------------------------------------------
                  OpenMS -- Open-Source Mass Spectrometry
--------------------------------------------------------------------------
Copyright The OpenMS Team -- Eberhard Karls University Tuebingen,
ETH Zurich, and Freie Universitaet Berlin 2002-2020.

This software is released under a three-clause BSD license:
 * Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.
 * Neither the name of any author or any participating institution
   may be used to endorse or promote products derived from this software
   without specific prior written permission.
For a full list of authors, refer to the file AUTHORS.
--------------------------------------------------------------------------
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL ANY OF THE AUTHORS OR THE CONTRIBUTING
INSTITUTIONS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

--------------------------------------------------------------------------
$Maintainer: Hannes Roest $
$Authors: Hannes Roest $
--------------------------------------------------------------------------
"""

"""
This program can be used to only compile the pyopenms.pyx to a cpp file using
Cython. This is an intermediate step in the pyOpenMS build process and it
should only be used for debugging.

It may be useful if something went wrong during the Cython compilation. One can
then try to fix the pyopenms.pyx file, run this script and see whether Cython
parses it.

Author: Hannes Roest
"""

# Py2/3 fix
try:
    import cPickle
except ImportError:
    import _pickle as cPickle

persisted_data_path = "include_dir.bin"
try:
    autowrap_include_dirs = cPickle.load(open(persisted_data_path, "rb"))
except IOError:
    print("The file include_dir.bin does not yet exist, please run setup.py first to create it.")

from Cython.Compiler.Main import compile, CompilationOptions
import Cython
print("Will try to compile with Cython version", Cython.__version__)

# Prepare options
print ("include:", autowrap_include_dirs)
options = dict(include_path=autowrap_include_dirs,
               #output_dir=".",
               #gdb_debug=True,
               cplus=True)

# Do Cython compile
import sys
out  = sys.argv[1]
options = CompilationOptions(**options)
compile(out, options=options)




from libcpp.vector cimport vector as libcpp_vector
from libcpp.set cimport set as libcpp_set
from libcpp cimport bool
from Types cimport *
from String cimport *
from EmpiricalFormula cimport *

cdef extern from "<OpenMS/CHEMISTRY/DigestionEnzymeRNA.h>" namespace "OpenMS":

    cdef cppclass DigestionEnzymeRNA:
        DigestionEnzymeRNA() nogil except + # wrap-ignore

        DigestionEnzymeRNA(DigestionEnzymeRNA) nogil except + # wrap-ignore

        bool setValueFromFile(const String & key, const String & value) nogil except +

        # sets the name of the Enzyme
        void setName(String name) nogil except +

        # returns the name of the Enzyme
        String getName() nogil except +

        # sets the synonyms
        void setSynonyms(libcpp_set[String] synonyms) nogil except +

        # adds a synonym
        void addSynonym(String synonym) nogil except +

        # returns the sysnonyms
        libcpp_set[String] getSynonyms() nogil except +

        # sets the name of the Enzyme as three letter code
        void setRegEx(String three_letter_code) nogil except +

        # returns the name of the Enzyme as three letter code
        String getRegEx() nogil except +

        # sets the regex description
        void setRegExDescription(String one_letter_code) nogil except +

        # returns the regex description
        String getRegExDescription() nogil except +

        # sets the 3' gain
        void setThreePrimeGain(String value) nogil except +

        # sets the 5' gain
        void setFivePrimeGain(String value) nogil except +

        # returns the 3' gain
        String getThreePrimeGain() nogil except +

        # returns the 5' gain
        String getFivePrimeGain() nogil except +

        # equality operator
        bool operator==(DigestionEnzymeRNA& Enzyme) nogil except +

        # inequality operator
        bool operator!=(DigestionEnzymeRNA& Enzyme) nogil except +

        # equality operator for cleavage regex
        bool operator==(EmpiricalFormula cleavage_regex) nogil except +

        # equality operator for cleavage regex
        bool operator!=(EmpiricalFormula cleavage_regex) nogil except +




from MSQuantifications cimport *
from String cimport *
from StringList cimport *

cdef extern from "<OpenMS/FORMAT/MzQuantMLFile.h>" namespace "OpenMS":

    cdef cppclass MzQuantMLFile:
        MzQuantMLFile() nogil except +

        void load(String, MSQuantifications &) nogil except+
        void store(String, MSQuantifications &) nogil except+

        bool isSemanticallyValid(String filename,
                                 StringList & errors,
                                 StringList & warnings) nogil except +





from libcpp.vector cimport vector as libcpp_vector
from DefaultParamHandler cimport *
from DeNovoIonScoring cimport *
from PeptideIdentification cimport *
from MSSpectrum cimport *
from Peak1D cimport *

cdef extern from "<OpenMS/ANALYSIS/DENOVO/DeNovoAlgorithm.h>" namespace "OpenMS":

    cdef cppclass DeNovoAlgorithm "OpenMS::DeNovoAlgorithm":
        # wrap-ignore
        # no-pxd-import
        DeNovoAlgorithm() nogil except +
        DeNovoAlgorithm(DeNovoAlgorithm) nogil except +
        void generateCandidates(libcpp_vector[ PeptideIdentification ] & candidates,
                                libcpp_vector[ libcpp_vector[ DeNovoIonScore ] ] &ion_scores,
                                MSExperiment &exp) nogil except +
        void generateCandidates(PeptideIdentification &candidates,
                                libcpp_vector[ DeNovoIonScore ] & ion_scores,
                                MSSpectrum & spec) nogil except +




from Types cimport *

cdef extern from "<OpenMS/ANALYSIS/OPENSWATH/MRMRTNormalizer.h>" namespace "OpenMS":

    cdef cppclass MRMRTNormalizer:
        pass

cdef extern from "<OpenMS/ANALYSIS/OPENSWATH/MRMRTNormalizer.h>" namespace "OpenMS::MRMRTNormalizer":

    libcpp_vector[libcpp_pair[double,double]] removeOutliersIterative(
            libcpp_vector[libcpp_pair[double,double]] & pairs,
            double rsq_limit,
            double coverage_limit, 
            bool use_chauvenet,
            libcpp_string outlier_detection_method 
            ) nogil except + # wrap-attach:MRMRTNormalizer

    libcpp_vector[libcpp_pair[double,double]] removeOutliersRANSAC(
            libcpp_vector[libcpp_pair[double,double]] & pairs,
            double rsq_limit,
            double coverage_limit, 
            size_t max_iterations,
            double max_rt_threshold,
            size_t sampling_size
            ) nogil except + # wrap-attach:MRMRTNormalizer

    double chauvenet_probability(libcpp_vector[ double ] residuals, int pos) nogil except + # wrap-attach:MRMRTNormalizer
    bool chauvenet(libcpp_vector[ double ] residuals, int pos) nogil except + # wrap-attach:MRMRTNormalizer

    bool computeBinnedCoverage(libcpp_pair[double,double] rtRange, 
                               libcpp_vector[libcpp_pair[double,double]] & pairs,
                               int nrBins, 
                               int minPeptidesPerBin,
                               int minBinsFilled) nogil except + # wrap-attach:MRMRTNormalizer







    property tableRows:
    
        def __set__(self, list tableRows):
            # libcpp_vector[ libcpp_vector[String] ]
            cdef libcpp_vector[libcpp_vector[_String]] * v0 = new libcpp_vector[libcpp_vector[_String]]()
            cdef libcpp_vector[_String] * v0_rec = new libcpp_vector[_String]()
            cdef bytes item0
            for tableRows_rec in tableRows:
                v0_rec.clear()
                for item0 in tableRows_rec:
                    v0_rec.push_back(_String(<char *>item0))
                v0.push_back(deref(v0_rec))
            self.inst.get().tableRows = deref(v0)
            del v0
            del v0_rec
    

        def __get__(self):
            c_res = self.inst.get().tableRows
            # libcpp_vector[ libcpp_vector[String] ]
            cdef replace = []
            cdef libcpp_vector[libcpp_vector[_String] ].iterator it = c_res.begin()
            cdef libcpp_vector[_String ].iterator it_inner
            while it != c_res.end():
                replace_inner = []
                it_inner = deref(it).begin()
                while it_inner != deref(it).end():
                    replace_inner.append(<char*>deref(it_inner).c_str())
                    inc(it_inner)
                replace.append(replace_inner)
                inc(it)
            return replace 



from Types cimport *
from libcpp cimport bool
from Types cimport *
from String cimport *

cdef extern from "<OpenMS/METADATA/PeptideEvidence.h>" namespace "OpenMS":
    cdef cppclass PeptideEvidence :

        PeptideEvidence() nogil except +
        PeptideEvidence(PeptideEvidence) nogil except +

        # const members
        ## int UNKNOWN_POSITION
        ## int N_TERMINAL_POSITION
        ## char UNKNOWN_AA
        ## char N_TERMINAL_AA
        ## char C_TERMINAL_AA

        void setStart(Int start) nogil except +
        Int getStart() nogil except +
        void setEnd(Int end) nogil except +
        Int getEnd() nogil except +
        void setAABefore(char rhs) nogil except +
        char getAABefore() nogil except +
        void setAAAfter(char rhs) nogil except +
        char getAAAfter() nogil except +
        void setProteinAccession(String s) nogil except +
        String getProteinAccession() nogil except +

        bool hasValidLimits() nogil except +

        bool operator==(PeptideEvidence & rhs) nogil except +
        bool operator!=(PeptideEvidence & rhs) nogil except +




#!/usr/bin/env python
# -*- coding: utf-8  -*-
"""
--------------------------------------------------------------------------
                  OpenMS -- Open-Source Mass Spectrometry
--------------------------------------------------------------------------
Copyright The OpenMS Team -- Eberhard Karls University Tuebingen,
ETH Zurich, and Freie Universitaet Berlin 2002-2020.

This software is released under a three-clause BSD license:
 * Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.
 * Neither the name of any author or any participating institution
   may be used to endorse or promote products derived from this software
   without specific prior written permission.
For a full list of authors, refer to the file AUTHORS.
--------------------------------------------------------------------------
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL ANY OF THE AUTHORS OR THE CONTRIBUTING
INSTITUTIONS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

--------------------------------------------------------------------------
$Maintainer: Hannes Roest $
$Authors: Hannes Roest $
--------------------------------------------------------------------------
"""
from __future__ import division
from __future__ import print_function

# Create simulated sonar scans for testing

from pyopenms import *

exp = MSExperiment()
# print(dir(exp))

# Create MS1 spectra
for rt_idx in range(50):
    sp = MSSpectrum()
    sp.setRT(rt_idx)
    sp.setMSLevel(1)
    for i in range(100):
        p = Peak1D()
        p.setMZ(100+i)
        p.setIntensity(100+i)
        sp.push_back(p)
    exp.addSpectrum(sp)


NR_RT_SAMPLES = 50
NR_SONAR_SP = 200
NR_SONAR_SP = 50
NR_PEAKS = 5

# Create MS2 spectra
for rt_idx in range(NR_RT_SAMPLES):
    # Base intensity is a single peak at 25 RT with 100 *i intensity spread across 9 SONAR scans
    # Second base intensity is a single peak at 10 RT with 100 *i intensity spread across 9 SONAR scans
    base_int = 100 - abs(25 - rt_idx)*(100)/25.0
    base_int_second = 100 - abs(10 - rt_idx)*(100)/40.0
    print("base int", base_int, abs(25 - rt_idx)*(100)/25.0  )

    for sonar_idx in range(NR_SONAR_SP):
        print("======================================= sonar", sonar_idx)
        sp = MSSpectrum()
        p = pyopenms.Precursor()
        p.setIsolationWindowLowerOffset(12)
        p.setIsolationWindowUpperOffset(12)
        target_mz = sonar_idx * 2.5 + 400
        p.setMZ(target_mz)
        sp.setPrecursors([p])

        sp.setRT(rt_idx)
        sp.setMSLevel(2)

        # peaks of a precursor at 412.5 m/z : 100, 101, 102, .. 100 + NR_PEAKS
        #  range from window 0 to 10
        for i in range(NR_PEAKS):
            if 412.5 > target_mz - 12 and 412.5 < target_mz + 12:
                p = Peak1D()
                p.setMZ(100+i)
                p.setIntensity(base_int * (i + 1) + sonar_idx)
                sp.push_back(p)
            else:
                # add noise data (6x less)
                p = Peak1D()
                p.setMZ(100+i)
                p.setIntensity(base_int * (i + 1)/ 6.0)
                sp.push_back(p)

        # peaks of a precursor at 462.5 m/z : 100, 101, 102, .. 100 + NR_PEAKS
        #  range from window 20 to 30
        for i in range(NR_PEAKS):
            if 462.5 > target_mz - 12 and 462.5 < target_mz + 12:
                p = Peak1D()
                p.setMZ(300+i)
                p.setIntensity(base_int_second * (i + 1))
                sp.push_back(p)
            else:
                # add noise data (6x less)
                p = Peak1D()
                p.setMZ(300+i)
                p.setIntensity(base_int_second * (i + 1)/ 6.0)
                sp.push_back(p)

        exp.addSpectrum(sp)
    # For debug:
    # break


f = MzMLFile()
pf = f.getOptions()
pf.setCompression(True)
f.setOptions(pf)
f.store('sonar.mzML', exp)





from Types cimport *
from libcpp cimport bool
from libcpp.map cimport map as libcpp_map
from libcpp.pair cimport pair as libcpp_pair
from libcpp.vector cimport vector as libcpp_vector
from String cimport *
from MSExperiment cimport *
from PeptideIdentification cimport *
from FileHandler cimport *

cdef extern from "<OpenMS/FORMAT/InspectOutfile.h>" namespace "OpenMS":
    
    cdef cppclass InspectOutfile "OpenMS::InspectOutfile":
        InspectOutfile() nogil except +
        InspectOutfile(InspectOutfile) nogil except +

        bool operator==(InspectOutfile & inspect_outfile) nogil except +

        libcpp_vector[ size_t ] load(const String & result_filename, 
                                     libcpp_vector[ PeptideIdentification ] & peptide_identifications,
                                     ProteinIdentification & protein_identification, double
                                     p_value_threshold, const String & database_filename) nogil except +

        libcpp_vector[ size_t ] getWantedRecords(const String & result_filename,
                                                 double p_value_threshold) nogil except +

        void compressTrieDB(const String & database_filename, const String &
                            index_filename, libcpp_vector[ size_t ] &
                            wanted_records, const String & snd_database_filename,
                            const String & snd_index_filename, bool append) nogil except +

        void generateTrieDB(const String & source_database_filename, const String &
                            database_filename, const String & index_filename, bool
                            append, const String species) nogil except +

        void getACAndACType(String line,
                            String & accession,
                            String & accession_type) nogil except +

        # TODO mixed, nested STL
        void getPrecursorRTandMZ(
            libcpp_vector[ libcpp_pair[ String, libcpp_vector[ libcpp_pair[ size_t, size_t ] ] ] ] & files_and_peptide_identification_with_scan_number,
            libcpp_vector[ PeptideIdentification ] & ids) nogil except + # wrap-ignore

        void getLabels(const String & source_database_filename,
                       String & ac_label,
                       String & sequence_start_label,
                       String & sequence_end_label,
                       String & comment_label,
                       String & species_label) nogil except +

        libcpp_vector[ size_t ] getSequences(const String & database_filename,
                                             libcpp_map[ size_t, size_t ] & wanted_records,
                                             libcpp_vector[ String ] & sequences) nogil except +

        void getExperiment(MSExperiment & exp, String & type_, const String & in_filename) nogil except +

        bool getSearchEngineAndVersion(const String & cmd_output, ProteinIdentification & protein_identification) nogil except +

        void readOutHeader(const String & filename,
                           const String & header_line,
                           Int & spectrum_file_column,
                           Int & scan_column,
                           Int & peptide_column,
                           Int & protein_column,
                           Int & charge_column,
                           Int & MQ_score_column,
                           Int & p_value_column,
                           Int & record_number_column,
                           Int & DB_file_pos_column,
                           Int & spec_file_pos_column,
                           Size & number_of_columns) nogil except +




from Types cimport *
from libcpp cimport bool
from TransitionTSVFile cimport *
from TargetedExperiment cimport *
from LightTargetedExperiment cimport *

cdef extern from "<OpenMS/ANALYSIS/OPENSWATH/TransitionPQPFile.h>" namespace "OpenMS":

    cdef cppclass TransitionPQPFile:

        TransitionPQPFile() nogil except +
        TransitionPQPFile(TransitionPQPFile) nogil except + #wrap-ignore

        void convertTargetedExperimentToPQP(char * filename, TargetedExperiment & targeted_exp) nogil except +
        void convertPQPToTargetedExperiment(char * filename, TargetedExperiment & targeted_exp, bool legacy_traml_id) nogil except +
        void convertPQPToTargetedExperiment(char * filename, LightTargetedExperiment & targeted_exp, bool legacy_traml_id) nogil except +

        # inherited from TransitionTSVFile
        # due to issues with Cython and overloaded inheritance
        void convertTargetedExperimentToTSV(char * filename, TargetedExperiment& targeted_exp) nogil except +

        void convertTSVToTargetedExperiment(char * filename, FileType filetype, TargetedExperiment& targeted_exp) nogil except +
        void convertTSVToTargetedExperiment(char * filename, FileType filetype, LightTargetedExperiment& targeted_exp) nogil except +

        void validateTargetedExperiment(TargetedExperiment targeted_exp) nogil except +



from Types cimport *
from libcpp.string cimport string as libcpp_string

# cannot wrap template argument std::istream
# typename Container = std::map<std::string, double>,
# cdef extern from "<OpenMS/CHEMISTRY/MASSDECOMPOSITION/IMS/IMSAlphabetParser.h>" namespace "OpenMS::ims":
#     
#     cdef cppclass IMSAlphabetParser: # [AlphabetElementType,Container,InputSource]:
#         IMSAlphabetParser(IMSAlphabetParser) nogil except + #wrap-ignore

#         void load(libcpp_string & fname) nogil except +
#         # ContainerType getElements() nogil except +
#         # void parse(InputSource & is_) nogil except +






    # NOTE: using shared_ptr for a singleton will lead to segfaults, use raw ptr instead
    # Provides the same interface as expected by autowrap (get/assign) but
    # simply holds the ptr and does not do any memory management.
    # see autowrap/data_files/autowrap/README.md for implementation
    # cdef AutowrapPtrHolder[_CrossLinksDB] inst

    def __init__(self):
      self.inst = AutowrapPtrHolder[_CrossLinksDB](_getInstance_CrossLinksDB())

    def __dealloc__(self):
      # Careful here, the wrapped ptr is a single instance and we should not
      # reset it, therefore use 'wrap-manual-dealloc'
      pass





from String cimport *
from Software cimport *
from MetaInfoInterface cimport *
from libcpp.vector cimport vector as libcpp_vector

cdef extern from "<OpenMS/METADATA/IonSource.h>" namespace "OpenMS":

    cdef cppclass IonSource(MetaInfoInterface):
        # wrap-inherits:
        #    MetaInfoInterface

        IonSource() nogil except +
        IonSource(IonSource) nogil except + # wrap-ignore

        # returns the ionization mode
        Polarity getPolarity() nogil except +
        # sets the ionization mode
        void setPolarity(Polarity polarity) nogil except +

        # returns the inlet type
        InletType getInletType() nogil except +
        # sets the  inlet type
        void setInletType(InletType inlet_type) nogil except +

        # returns the ionization method
        IonizationMethod getIonizationMethod() nogil except +
        # sets the ionization method
        void setIonizationMethod(IonizationMethod ionization_type) nogil except +

        #     @brief returns the position of this part in the whole Instrument.
        #     Order can be ignored, as long the instrument has this default setup:
        #     - one ion source
        #     - one or many mass analyzers
        #     - one ion detector
        #     For more complex instruments, the order should be defined.
        Int getOrder() nogil except +
        # sets the order
        void setOrder(Int order) nogil except +

cdef extern from "<OpenMS/METADATA/IonSource.h>" namespace "OpenMS::IonSource":

    cdef enum Polarity:
        # wrap-attach:
        #     IonSource
        POLNULL, POSITIVE, NEGATIVE, SIZE_OF_POLARITY

    cdef enum InletType:
        # wrap-attach:
        #     IonSource
        INLETNULL,                                                        #]Unknown
        DIRECT,                                                               #]Direct
        BATCH,                                                                #]Batch (e.g. in MALDI)
        CHROMATOGRAPHY,                                               #]Chromatography (liquid)
        PARTICLEBEAM,                                                     #]Particle beam
        MEMBRANESEPARATOR,                                        #]Membrane sparator
        OPENSPLIT,                                                        #]Open split
        JETSEPARATOR,                                                     #]Jet separator
        SEPTUM,                                                               #]Septum
        RESERVOIR,                                                        #]Reservoir
        MOVINGBELT,                                                       #]Moving belt
        MOVINGWIRE,                                                       #]Moving wire
        FLOWINJECTIONANALYSIS,                                #]Flow injection analysis
        ELECTROSPRAYINLET,                                        #]Electro spray
        THERMOSPRAYINLET,                                             #]Thermo spray
        INFUSION,                                                             #]Infusion
        CONTINUOUSFLOWFASTATOMBOMBARDMENT,        #]Continuous flow fast atom bombardment
        INDUCTIVELYCOUPLEDPLASMA,                             #]Inductively coupled plasma
        MEMBRANE,                                                             #]Membrane inlet
        NANOSPRAY,                                                        #]Nanospray inlet
        SIZE_OF_INLETTYPE


    # ionization method
    cdef enum IonizationMethod:
        # wrap-attach:
        #     IonSource
        IONMETHODNULL,        #]Unknown
        ESI,                              #]electrospray ionisation
        EI,                               #]electron ionization
        CI,                               #]chemical ionisation
        FAB,                              #]fast atom bombardment
        TSP,                              #]thermospray
        LD,                               #]laser desorption
        FD,                               #]field desorption
        FI,                               #]flame ionization
        PD,                               #]plasma desorption
        SI,                               #]secondary ion MS
        TI,                               #]thermal ionization
        API,                              #]atmospheric pressure ionisation
        ISI,                              #<
        CID,                              #]collsion induced decomposition
        CAD,                              #]collsion activated decomposition
        HN,                               #<
        APCI,                             #]atmospheric pressure chemical ionization
        APPI,                             #]atmospheric pressure photo ionization
        ICP,                              #]inductively coupled plasma
        NESI,                             #]Nano electrospray ionization
        MESI,                             #]Micro electrospray ionization
        SELDI,                        #]Surface enhanced laser desorption ionization
        SEND,                             #]Surface enhanced neat desorption
        FIB,                              #]Fast ion bombardment
        MALDI,                        #]Matrix-assisted laser desorption ionization
        MPI,                              #]Multiphoton ionization
        DI,                               #]desorption ionization
        FA,                               #]flowing afterglow
        FII,                              #]field ionization
        GD_MS,                        #]glow discharge ionization
        NICI,                             #]negative ion chemical ionization
        NRMS,                             #]neutralization reionization mass spectrometry
        PI,                               #]photoionization
        PYMS,                             #]pyrolysis mass spectrometry
        REMPI,                        #]resonance enhanced multiphoton ionization
        AI,                               #]adiabatic ionization
        ASI,                              #]associative ionization
        AD,                               #]autodetachment
        AUI,                              #]autoionization
        CEI,                              #]charge exchange ionization
        CHEMI,                        #]chemi-ionization
        DISSI,                        #]dissociative ionization
        LSI,                              #]liquid secondary ionization
        PEI,                              #]penning ionization
        SOI,                              #]soft ionization
        SPI,                              #]spark ionization
        SUI,                              #]surface ionization
        VI,                               #]vertical ionization
        AP_MALDI,                     #]atmospheric pressure matrix-assisted laser desorption ionization
        SILI,                             #]desorption/ionization on silicon
        SALDI,                        #]surface-assisted laser desorption ionization
        SIZE_OF_IONIZATIONMETHOD



from Types cimport *
from libcpp cimport bool
from Types cimport *
from DataValue cimport *
from String cimport *
from Map cimport *
from ParamEntry cimport *

cdef extern from "<OpenMS/DATASTRUCTURES/Param.h>" namespace "OpenMS::Param":
    
    cdef cppclass ParamNode "OpenMS::Param::ParamNode":
        ParamNode() nogil except +
        ParamNode(ParamNode) nogil except + #wrap-ignore

        String name
        String description
        libcpp_vector[ ParamEntry ] entries
        libcpp_vector[ ParamNode ] nodes
        ParamNode(const String & n, const String & d) nogil except +
        bool operator==(ParamNode & rhs) nogil except +
        # EntryIterator findEntry(const String & name) nogil except +
        # NodeIterator findNode(const String & name) nogil except +
        ParamNode * findParentOf(const String & name) nogil except +
        ParamEntry * findEntryRecursive(const String & name) nogil except +
        void insert(ParamNode & node, const String & prefix) nogil except +
        void insert(ParamEntry & entry, const String & prefix) nogil except +
        Size size() nogil except +
        String suffix(const String & key) nogil except +




from Types cimport *
from libcpp cimport bool
from libcpp.pair cimport pair as libcpp_pair
# from TraceFitter cimport *
from Peak1D cimport *
from FeatureFinderAlgorithmPickedHelperStructs cimport *

cdef extern from "<OpenMS/TRANSFORMATIONS/FEATUREFINDER/GaussTraceFitter.h>" namespace "OpenMS":

    cdef cppclass GaussTraceFitter:
        GaussTraceFitter() nogil except +
        GaussTraceFitter(GaussTraceFitter) nogil except +
        void fit(MassTraces& traces) nogil except +
        double getLowerRTBound() nogil except +
        double getUpperRTBound() nogil except +
        double getHeight() nogil except +
        double getCenter() nogil except +
        double getFWHM() nogil except +
        double getSigma() nogil except +
        bool checkMaximalRTSpan(double max_rt_span) nogil except +
        bool checkMinimalRTSpan(libcpp_pair[ double, double ] & rt_bounds, double min_rt_span) nogil except +
        double computeTheoretical(MassTrace & trace, Size k) nogil except +
        double getArea() nogil except +
        String getGnuplotFormula(MassTrace & trace, char function_name, double baseline, double rt_shift) nogil except +
        double getValue(double rt) nogil except +




from Types cimport *
from IsobaricQuantitationMethod cimport *

cdef extern from "<OpenMS/ANALYSIS/QUANTITATION/TMTTenPlexQuantitationMethod.h>" namespace "OpenMS":

    cdef cppclass TMTTenPlexQuantitationMethod(IsobaricQuantitationMethod) :
        # wrap-inherits:
        #  IsobaricQuantitationMethod
        TMTTenPlexQuantitationMethod() nogil except +
        TMTTenPlexQuantitationMethod(TMTTenPlexQuantitationMethod) nogil except +




from Types cimport *
from libcpp cimport bool
from libcpp.vector cimport vector as libcpp_vector
from IMSIsotopeDistribution cimport *

cdef extern from "<OpenMS/CHEMISTRY/MASSDECOMPOSITION/IMS/Weights.h>" namespace "OpenMS::ims::Weights":

    ctypedef double alphabet_mass_type
    ctypedef long unsigned int weight_type
    ctypedef libcpp_vector[weight_type] weights_type

cdef extern from "<OpenMS/CHEMISTRY/MASSDECOMPOSITION/IMS/Weights.h>" namespace "OpenMS::ims":
    
    cdef cppclass IMSWeights "OpenMS::ims::Weights":
        IMSWeights() nogil except +
        IMSWeights(IMSWeights) nogil except +
        # IMSWeights(alphabet_masses_type & masses, alphabet_mass_type precision) nogil except +
        size_type size() nogil except +
        weight_type getWeight(size_type i) nogil except +
        void setPrecision(alphabet_mass_type precision) nogil except +
        alphabet_mass_type getPrecision() nogil except +
        # weight_type operator[](size_type i) nogil except +
        weight_type back() nogil except +
        alphabet_mass_type getAlphabetMass(size_type i) nogil except +
        alphabet_mass_type getParentMass(libcpp_vector[ unsigned int ] & decomposition) nogil except +
        void swap(size_type index1, size_type index2) nogil except +
        bool divideByGCD() nogil except +
        alphabet_mass_type getMinRoundingError() nogil except +
        alphabet_mass_type getMaxRoundingError() nogil except +




import unittest
import os

import pyopenms

class TestSpectrumAccessOpenMS(unittest.TestCase):

    def setUp(self):
        dirname = os.path.dirname(os.path.abspath(__file__))
        self.filename = os.path.join(dirname, "test2.mzML").encode()

    def test_readfile_content(self):
        exp = pyopenms.MSExperiment()
        pyopenms.MzMLFile().load(self.filename, exp)
        saccess = pyopenms.SpectrumAccessOpenMS(exp)
        spectrum = saccess.getSpectrumById(0)
        mz = spectrum.getMZArray()
        intensity = spectrum.getIntensityArray()

        self.assertAlmostEqual(mz[0], 350.0000305)
        self.assertAlmostEqual(intensity[0], 0.0)
        self.assertAlmostEqual(mz[10], 358.075134277)
        self.assertAlmostEqual(intensity[10], 9210.931640625)

class TestSpectrumAccessOpenMSInMemory(unittest.TestCase):

    def setUp(self):
        dirname = os.path.dirname(os.path.abspath(__file__))
        self.filename = os.path.join(dirname, "test2.mzML").encode()

    def test_readfile_content(self):
        exp = pyopenms.MSExperiment()
        pyopenms.MzMLFile().load(self.filename, exp)

        # Create in memory access
        saccess_ = pyopenms.SpectrumAccessOpenMS(exp)
        saccess = pyopenms.SpectrumAccessOpenMSInMemory(saccess_)

        spectrum = saccess.getSpectrumById(0)
        mz = spectrum.getMZArray()
        intensity = spectrum.getIntensityArray()

        self.assertAlmostEqual(mz[0], 350.0000305)
        self.assertAlmostEqual(intensity[0], 0.0)
        self.assertAlmostEqual(mz[10], 358.075134277)
        self.assertAlmostEqual(intensity[10], 9210.931640625)

class TestSpectrumAccessSwathMap(unittest.TestCase):

    def setUp(self):
        dirname = os.path.dirname(os.path.abspath(__file__))
        self.filename = os.path.join(dirname, "test2.mzML").encode()

    def test_swathmap_openms(self):
        exp = pyopenms.MSExperiment()
        pyopenms.MzMLFile().load(self.filename, exp)
        saccess = pyopenms.SpectrumAccessOpenMS(exp)

        swmap = pyopenms.SwathMap()
        swmap.lower = 400
        swmap.center = 412.5
        swmap.upper = 425
        swmap.ms1 = False

        data = swmap.getSpectrumPtr()
        assert data is None

        # Now we should have data in there
        swmap.setSpectrumPtr(saccess)
        data = swmap.getSpectrumPtr()
        assert data is not None

        swmap.setSpectrumPtr(saccess)

        data = swmap.getSpectrumPtr()
        assert data is not None

        spectrum = data.getSpectrumById(0)
        mz = spectrum.getMZArray()
        intensity = spectrum.getIntensityArray()

        self.assertAlmostEqual(mz[0], 350.0000305)
        self.assertAlmostEqual(intensity[0], 0.0)
        self.assertAlmostEqual(mz[10], 358.075134277)
        self.assertAlmostEqual(intensity[10], 9210.931640625)

    def test_readfile_contentInMemory(self):
        exp = pyopenms.MSExperiment()
        pyopenms.MzMLFile().load(self.filename, exp)

        # Create in memory access
        saccess_ = pyopenms.SpectrumAccessOpenMS(exp)
        saccess = pyopenms.SpectrumAccessOpenMSInMemory(saccess_)

        swmap = pyopenms.SwathMap()
        swmap.lower = 400
        swmap.center = 412.5
        swmap.upper = 425
        swmap.ms1 = False

        data = swmap.getSpectrumPtr()
        assert data is None

        swmap.setSpectrumPtr(saccess_)

        data = swmap.getSpectrumPtr()
        assert data is not None

        spectrum = data.getSpectrumById(0)
        mz = spectrum.getMZArray()
        intensity = spectrum.getIntensityArray()

        self.assertAlmostEqual(mz[0], 350.0000305)
        self.assertAlmostEqual(intensity[0], 0.0)
        self.assertAlmostEqual(mz[10], 358.075134277)
        self.assertAlmostEqual(intensity[10], 9210.931640625)

if __name__ == '__main__':
    unittest.main()



from Types cimport *
from libcpp cimport bool
from ConsensusMap cimport *

cdef extern from "<OpenMS/ANALYSIS/MAPMATCHING/MapAlignmentEvaluationAlgorithm.h>" namespace "OpenMS":
    
    cdef cppclass MapAlignmentEvaluationAlgorithm "OpenMS::MapAlignmentEvaluationAlgorithm":
        # wrap-ignore
        # ABSTRACT class
        # no-pxd-import
        MapAlignmentEvaluationAlgorithm() nogil except +
        MapAlignmentEvaluationAlgorithm(MapAlignmentEvaluationAlgorithm) nogil except + #wrap-ignore

        # NAMESPACE # void evaluate(ConsensusMap & conensus_map_in, ConsensusMap & consensus_map_gt, double & rt_dev, double & mz_dev, Peak2D::IntensityType & int_dev, bool use_charge, double & out) nogil except +
        # NAMESPACE # bool isSameHandle(FeatureHandle & lhs, FeatureHandle & rhs, double & rt_dev, double & mz_dev, Peak2D::IntensityType & int_dev, bool use_charge) nogil except +
        void registerChildren() nogil except +




from libcpp.vector cimport vector as libcpp_vector
from libcpp cimport bool
from MSSpectrum cimport *
from Map cimport *
from ConsensusMap cimport *
from MSExperiment cimport *
from ChromatogramPeak cimport *
from Peak1D cimport *
from PeptideIdentification cimport *
from DefaultParamHandler cimport *
from ProgressLogger cimport *
from DPosition cimport DPosition2


ctypedef libcpp_vector[DPosition2] SeedList

cdef extern from "<OpenMS/TRANSFORMATIONS/FEATUREFINDER/SeedListGenerator.h>" namespace "OpenMS":

    cdef cppclass SeedListGenerator:

        SeedListGenerator()                    nogil except +
        SeedListGenerator(SeedListGenerator &) nogil except +

        void generateSeedList(MSExperiment exp, libcpp_vector[DPosition2] & seeds) nogil except +
        void generateSeedList(libcpp_vector[PeptideIdentification] & peptides, libcpp_vector[DPosition2] & seeds, bool use_peptide_mass) nogil except +
        # TODO map with UInt64
        void generateSeedList(ConsensusMap & consensus, Map[unsigned long, libcpp_vector[DPosition2] ] & seeds) nogil except +  # wrap-ignore

        # TODO nested STL
        # void generateSeedLists(ConsensusMap & consensus, Map[ UInt64, libcpp_vector[ DPosition2] ] & seed_lists) nogil except +
        void convertSeedList(libcpp_vector[ DPosition2] & seeds, FeatureMap & features) nogil except +
        void convertSeedList(FeatureMap & features, libcpp_vector[ DPosition2] & seeds) nogil except +



from Types cimport *
from FeatureMap cimport *
from MSExperiment cimport *

cdef extern from "<OpenMS/KERNEL/PeakIndex.h>" namespace "OpenMS":
    
    cdef cppclass PeakIndex "OpenMS::PeakIndex":
        PeakIndex() nogil except +
        PeakIndex(PeakIndex) nogil except + #wrap-ignore
        Size peak
        Size spectrum
        PeakIndex(Size peak) nogil except +
        PeakIndex(Size spectrum, Size peak) nogil except +
        bool isValid() nogil except +
        void clear() nogil except +
        Feature getFeature(FeatureMap & map_) nogil except +
        Peak1D getPeak(MSExperiment & map_) nogil except +
        MSSpectrum getSpectrum(MSExperiment & map_) nogil except +
        bool operator==(PeakIndex & rhs) nogil except +
        bool operator!=(PeakIndex & rhs) nogil except +




from Types cimport *
from SampleTreatment cimport *
from String cimport *

cdef extern from "<OpenMS/METADATA/Modification.h>" namespace "OpenMS":
    
    cdef cppclass Modification(SampleTreatment) :
        # wrap-inherits:
        #  SampleTreatment
        Modification() nogil except +
        Modification(Modification) nogil except +

        # bool operator==(SampleTreatment & rhs) nogil except +

        # SampleTreatment * clone() nogil except +

        String getReagentName() nogil except +
        void setReagentName(const String & reagent_name) nogil except +

        double getMass() nogil except +
        void setMass(double mass) nogil except +

        Modification_SpecificityType getSpecificityType() nogil except +
        void setSpecificityType(Modification_SpecificityType & specificity_type) nogil except +

        String getAffectedAminoAcids() nogil except +
        void setAffectedAminoAcids(const String & affected_amino_acids) nogil except +

cdef extern from "<OpenMS/METADATA/Modification.h>" namespace "OpenMS::Modification":
    cdef enum Modification_SpecificityType "OpenMS::Modification::SpecificityType":
        #wrap-attach:
        #    Modification
        AA
        AA_AT_CTERM
        AA_AT_NTERM
        CTERM
        NTERM
        SIZE_OF_SPECIFICITYTYPE




from SourceFile cimport *
from DateTime cimport *
from FileTypes cimport *
from String cimport *
from libcpp.vector cimport vector as libcpp_vector

cdef extern from "<OpenMS/METADATA/DocumentIdentifier.h>" namespace "OpenMS":

    cdef cppclass DocumentIdentifier:

        DocumentIdentifier() nogil except +
        DocumentIdentifier(DocumentIdentifier) nogil except + # wrap-ignore

        # set document identifier (e.g. an LSID)
        void setIdentifier(String id) nogil except +

        # retrieve document identifier (e.g. an LSID)
        String getIdentifier() nogil except +

        # exchange content with @p from -> gets overwritten later
        # this does not work since some derived classes overwrite this (e.g. MSExperiment)
        ## void swap(DocumentIdentifier from_) nogil except + 

        # set the file_type according to the type of the file loaded from (see FileHandler::Type) preferably done whilst loading
        void setLoadedFileType(String file_name) nogil except +

        # get the file_type (e.g. featureXML, consensusXML, mzData, mzXML, mzML, ...) of the file loaded from
        int getLoadedFileType() nogil except +

        # set the file_name_ according to absolute path of the file loaded from preferably done whilst loading
        void setLoadedFilePath(String file_name) nogil except +
        # get the file_name_ which is the absolute path to the file loaded from
        String getLoadedFilePath() nogil except +

        # Errors in C++
        # void swap(DocumentIdentifier & from_) nogil except +




from Types cimport *
from libcpp cimport bool
from libcpp.set cimport set as libcpp_set
from libcpp.vector cimport vector as libcpp_vector
from String cimport *
from EmpiricalFormula cimport *

cdef extern from "<OpenMS/CHEMISTRY/ResidueModification.h>" namespace "OpenMS":

    cdef cppclass ResidueModification "OpenMS::ResidueModification":
        # wrap-hash:
        #   getFullId().c_str()

        ResidueModification() nogil except +
        ResidueModification(ResidueModification) nogil except +

        bool operator==(ResidueModification & modification) nogil except +
        bool operator!=(ResidueModification & modification) nogil except +

        void setId(const String & id_) nogil except +
        String  getId() nogil except +
        void setFullId(const String & full_id) nogil except +
        String  getFullId() nogil except +

        Int getUniModRecordId() nogil except +
        void setUniModRecordId(Int id_) nogil except +
        String  getUniModAccession() nogil except +

        void setPSIMODAccession(const String & id_) nogil except +
        String getPSIMODAccession() nogil except +
        void setFullName(const String & full_name) nogil except +
        String getFullName() nogil except +
        void setName(const String & name) nogil except +
        String getName() nogil except +
        void setTermSpecificity(TermSpecificity term_spec) nogil except +
        void setTermSpecificity(const String & name) nogil except +

        TermSpecificity getTermSpecificity() nogil except +
        String getTermSpecificityName(TermSpecificity ) nogil except +
        void setOrigin(char origin) nogil except +
        char  getOrigin() nogil except +

        void setSourceClassification(const String & classification) nogil except +
        void setSourceClassification(SourceClassification classification) nogil except +
        SourceClassification getSourceClassification() nogil except +
        String getSourceClassificationName(SourceClassification classification) nogil except +

        void setAverageMass(double mass) nogil except +
        double getAverageMass() nogil except +
        void setMonoMass(double mass) nogil except +
        double getMonoMass() nogil except +
        void setDiffAverageMass(double mass) nogil except +
        double getDiffAverageMass() nogil except +
        void setDiffMonoMass(double mass) nogil except +
        double getDiffMonoMass() nogil except +

        void setFormula(const String & composition) nogil except +
        String  getFormula() nogil except +
        void setDiffFormula(EmpiricalFormula & diff_formula) nogil except +
        EmpiricalFormula  getDiffFormula() nogil except +

        void setSynonyms(libcpp_set[ String ] & synonyms) nogil except +
        void addSynonym(const String & synonym) nogil except +
        libcpp_set[ String ] getSynonyms() nogil except +

        void setNeutralLossDiffFormulas(libcpp_vector[ EmpiricalFormula ] & diff_formulas) nogil except +
        libcpp_vector[ EmpiricalFormula ] getNeutralLossDiffFormulas() nogil except +

        void setNeutralLossMonoMasses(libcpp_vector[ double ] mono_masses) nogil except +
        libcpp_vector[ double ] getNeutralLossMonoMasses() nogil except +
        void setNeutralLossAverageMasses(libcpp_vector[ double ] average_masses) nogil except +
        libcpp_vector[ double ] getNeutralLossAverageMasses() nogil except +

        bool hasNeutralLoss() nogil except +
        bool isUserDefined() nogil except +

cdef extern from "<OpenMS/CHEMISTRY/ResidueModification.h>" namespace "OpenMS::ResidueModification":
    cdef enum TermSpecificity "OpenMS::ResidueModification::TermSpecificity":
        #wrap-attach:
        #    ResidueModification
        ANYWHERE
        C_TERM
        N_TERM
        PROTEIN_C_TERM
        PROTEIN_N_TERM
        NUMBER_OF_TERM_SPECIFICITY

cdef extern from "<OpenMS/CHEMISTRY/ResidueModification.h>" namespace "OpenMS::ResidueModification":
    cdef enum SourceClassification "OpenMS::ResidueModification::SourceClassification":
        #wrap-attach:
        #    ResidueModification
        ARTIFACT
        HYPOTHETICAL
        NATURAL
        POSTTRANSLATIONAL
        MULTIPLE
        CHEMICAL_DERIVATIVE
        ISOTOPIC_LABEL
        PRETRANSLATIONAL
        OTHER_GLYCOSYLATION
        NLINKED_GLYCOSYLATION
        AA_SUBSTITUTION
        OTHER
        NONSTANDARD_RESIDUE
        COTRANSLATIONAL
        OLINKED_GLYCOSYLATION
        UNKNOWN
        NUMBER_OF_SOURCE_CLASSIFICATIONS



import unittest
import os

import pyopenms

import env

eps = 2

class TestMRMFeatureFinderScoring(unittest.TestCase):

    def setUp(self):
        self.dirname = os.path.dirname(os.path.abspath(__file__))
        self.testdirname = os.path.join(env.OPEN_MS_SRC, "src/tests/topp")
        # set up files
        self.chromatograms = os.path.join(self.testdirname, "OpenSwathAnalyzer_1_input_chrom.mzML").encode()
        self.tramlfile = os.path.join(self.testdirname, "OpenSwathAnalyzer_1_input.TraML").encode()

    def test_run_mrmfeaturefinder(self):

        # load chromatograms
        chromatograms = pyopenms.MSExperiment()
        fh = pyopenms.FileHandler()
        fh.loadExperiment(self.chromatograms, chromatograms)

        # load TraML file
        targeted = pyopenms.TargetedExperiment();
        tramlfile = pyopenms.TraMLFile();
        tramlfile.load(self.tramlfile, targeted);

        # Create empty files as input and finally as output
        empty_swath = pyopenms.MSExperiment()
        trafo = pyopenms.TransformationDescription()
        output = pyopenms.FeatureMap();

        # set up featurefinder and run
        featurefinder = pyopenms.MRMFeatureFinderScoring()
        featurefinder.pickExperiment(chromatograms, output, targeted, trafo, empty_swath)

        self.assertAlmostEqual(output.size(), 3)
        self.assertAlmostEqual(output[0].getRT(), 3119.092041015, eps)
        self.assertAlmostEqual(output[0].getIntensity(), 3614.99755859375, eps)
        self.assertAlmostEqual(output[0].getMetaValue(b"var_xcorr_shape_weighted"), 0.997577965259552, eps)
        self.assertAlmostEqual(output[0].getMetaValue(b"sn_ratio"), 86.00413513183594, eps)

if __name__ == '__main__':
    unittest.main()



import pyopenms


def test_convex_hull_0():
    ch = pyopenms.ConvexHull2D()

def test_convex_hull_1():
    ch = pyopenms.ConvexHull2D()

    points = [ [1.0, 1.0], [2.0, 1.0], [1.0, 2.0], [2.0, 2.0]]
    for p in points:
        ch.addPoint(p)

    assert ch.encloses([1.5, 1.5])
    assert not ch.encloses([.5, 1.5])

    hp = ch.getHullPoints()
    # order may change, so we use sort here:
    assert  sorted(hp.tolist()) == sorted(points)

    ch.setHullPoints(hp)
    hp = ch.getHullPoints()
    # order may change, so we use sorted here:
    assert  sorted(hp.tolist()) == sorted(points)

    ch.addPoints(hp+1.0)
    ch.addPoints(hp)
    hp = ch.getHullPoints()
    assert hp.shape == (6, 2)

    bb = ch.getBoundingBox()
    assert bb.minPosition() == [1.0, 1.0]
    assert bb.maxPosition() == [3.0, 3.0]



from Types cimport *
from IsobaricQuantitationMethod cimport *

cdef extern from "<OpenMS/ANALYSIS/QUANTITATION/ItraqEightPlexQuantitationMethod.h>" namespace "OpenMS":
    
    cdef cppclass ItraqEightPlexQuantitationMethod(IsobaricQuantitationMethod) :
        # wrap-inherits:
        #  IsobaricQuantitationMethod
        ItraqEightPlexQuantitationMethod() nogil except +
        ItraqEightPlexQuantitationMethod(ItraqEightPlexQuantitationMethod) nogil except +




from Types cimport *
from libcpp.vector cimport vector as libcpp_vector
from libcpp.pair cimport pair as libcpp_pair
from libcpp cimport bool
from libcpp.string cimport string as libcpp_string
from XLPrecursor cimport *
from AASeqWithMass cimport *
from DoubleList cimport *
from StringList cimport *
from IntList cimport *
from ResidueModification cimport *
from FASTAFile cimport *
from EnzymaticDigestion cimport *
from ProteinProteinCrossLink cimport *
from CrossLinkSpectrumMatch cimport *
from PeptideHit cimport *
from PeptideIdentification cimport *
from MSSpectrum cimport *
from MSExperiment cimport *
from EnzymaticDigestion cimport *
from DataArrays cimport *
from ModifiedPeptideGenerator cimport *


cdef extern from "<OpenMS/ANALYSIS/XLMS/OPXLHelper.h>" namespace "OpenMS":

    cdef cppclass OPXLHelper:

        OPXLHelper(OPXLHelper) nogil except +
        OPXLHelper() nogil except +

        libcpp_vector[ XLPrecursor ] enumerateCrossLinksAndMasses(libcpp_vector[ AASeqWithMass ]  peptides,
                                                                  double cross_link_mass_light,
                                                                  DoubleList cross_link_mass_mono_link,
                                                                  StringList cross_link_residue1,
                                                                  StringList cross_link_residue2,
                                                                  libcpp_vector[ double ]& spectrum_precursors,
                                                                  libcpp_vector[ int ]& precursor_correction_positions,
                                                                  double precursor_mass_tolerance,
                                                                  bool precursor_mass_tolerance_unit_ppm)  nogil except +

        libcpp_vector[ AASeqWithMass ] digestDatabase(libcpp_vector[ FASTAEntry ] fasta_db,
                                                      EnzymaticDigestion digestor,
                                                      Size min_peptide_length,
                                                      StringList cross_link_residue1,
                                                      StringList cross_link_residue2,
                                                      ModifiedPeptideGenerator_MapToResidueType& fixed_modifications,
                                                      ModifiedPeptideGenerator_MapToResidueType& variable_modifications,
                                                      Size max_variable_mods_per_peptide) nogil except +

        libcpp_vector[ ProteinProteinCrossLink ] buildCandidates(libcpp_vector[ XLPrecursor ]& candidates,
                                                                 libcpp_vector[ int ]& precursor_corrections,
                                                                 libcpp_vector[ int ]& precursor_correction_positions,
                                                                 libcpp_vector[ AASeqWithMass ]& peptide_masses,
                                                                 const StringList& cross_link_residue1,
                                                                 const StringList& cross_link_residue2,
                                                                 double cross_link_mass,
                                                                 DoubleList cross_link_mass_mono_link,
                                                                 libcpp_vector[ double ]& spectrum_precursor_vector,
                                                                 libcpp_vector[ double ]& allowed_error_vector,
                                                                 String cross_link_name) nogil except +


        void buildFragmentAnnotations(libcpp_vector[ PeptideHit_PeakAnnotation ]& frag_annotations,
                                      libcpp_vector[ libcpp_pair[ size_t, size_t ] ] matching,
                                      MSSpectrum theoretical_spectrum,
                                      MSSpectrum experiment_spectrum) nogil except +

        void buildPeptideIDs(libcpp_vector[ PeptideIdentification ]& peptide_ids,
                             libcpp_vector[ CrossLinkSpectrumMatch ] top_csms_spectrum,
                             libcpp_vector[ libcpp_vector[ CrossLinkSpectrumMatch ] ]& all_top_csms,
                             Size all_top_csms_current_index,
                             MSExperiment spectra,
                             Size scan_index,
                             Size scan_index_heavy) nogil except +

        void addProteinPositionMetaValues(libcpp_vector[ PeptideIdentification ]& peptide_ids) nogil except +

        void addXLTargetDecoyMV(libcpp_vector[ PeptideIdentification ]& peptide_ids) nogil except +

        void addBetaAccessions(libcpp_vector[ PeptideIdentification ]& peptide_ids) nogil except +

        void removeBetaPeptideHits(libcpp_vector[ PeptideIdentification ]& peptide_ids) nogil except +

        void addPercolatorFeatureList(ProteinIdentification& prot_id) nogil except +

        void computeDeltaScores(libcpp_vector[ PeptideIdentification ]& peptide_ids) nogil except +

        libcpp_vector[ PeptideIdentification ] combineTopRanksFromPairs(libcpp_vector[ PeptideIdentification ]& peptide_ids, Size number_top_hits) nogil except +

        libcpp_vector[ ProteinProteinCrossLink ] collectPrecursorCandidates(IntList precursor_correction_steps,
                                                                            double precursor_mass,
                                                                            double precursor_mass_tolerance,
                                                                            bool precursor_mass_tolerance_unit_ppm,
                                                                            libcpp_vector[ AASeqWithMass ] filtered_peptide_masses,
                                                                            double cross_link_mass,
                                                                            DoubleList cross_link_mass_mono_link,
                                                                            StringList cross_link_residue1,
                                                                            StringList cross_link_residue2,
                                                                            String cross_link_name,
                                                                            bool use_sequence_tags,
                                                                            const libcpp_vector[ libcpp_string ]& tags) nogil except +

        double computePrecursorError(CrossLinkSpectrumMatch csm, double precursor_mz, int precursor_charge) nogil except +

        void isoPeakMeans(CrossLinkSpectrumMatch& csm,
                          IntegerDataArray& num_iso_peaks_array,
                          libcpp_vector[ libcpp_pair[ size_t, size_t ] ]& matched_spec_linear_alpha,
                          libcpp_vector[ libcpp_pair[ size_t, size_t ] ]& matched_spec_linear_beta,
                          libcpp_vector[ libcpp_pair[ size_t, size_t ] ]& matched_spec_xlinks_alpha,
                          libcpp_vector[ libcpp_pair[ size_t, size_t ] ]& matched_spec_xlinks_beta) nogil except +



from Types cimport *
from MSSpectrum cimport *
from MSChromatogram cimport *
from ExperimentalSettings cimport *

cdef extern from "<OpenMS/FORMAT/DATAACCESS/MSDataAggregatingConsumer.h>" namespace "OpenMS":
    
    cdef cppclass MSDataAggregatingConsumer :

        MSDataAggregatingConsumer(MSDataAggregatingConsumer) nogil except + #wrap-ignore

        void consumeSpectrum(MSSpectrum & s) nogil except +
        void consumeChromatogram(MSChromatogram & ) nogil except +

        void setExpectedSize(Size expectedSpectra, Size expectedChromatograms) nogil except +
        void setExperimentalSettings(ExperimentalSettings & exp) nogil except +





import sys
import pyopenms

def getTransitionGroup(exp, targeted, key, trgr_ids, chrom_map):
    r = pyopenms.MRMTransitionGroupCP()
    for tr in trgr_ids:
        transition = targeted.getTransitions()[tr]
        chrom_idx = chrom_map[ transition.getNativeID() ]
        chrom = exp.getChromatograms()[ chrom_idx ]
        chrom.setMetaValue("product_mz", transition.getProductMZ() )
        chrom.setMetaValue("precursor_mz", transition.getPrecursorMZ() )
        r.addTransition( transition, transition.getNativeID() )
        r.addChromatogram( chrom, chrom.getNativeID() )
    return r


def algorithm(exp, targeted, picker):

    output = pyopenms.FeatureMap()

    chrom_map = {}
    pepmap = {}
    trmap = {}
    for i, chrom in enumerate(exp.getChromatograms()):
        chrom_map[ chrom.getNativeID() ] = i
    for i, pep in enumerate(targeted.getPeptides() ):
        pepmap[ pep.id ] = i
    for i, tr in enumerate(targeted.getTransitions() ):
        tmp = trmap.get( tr.getPeptideRef() , [])
        tmp.append( i )
        trmap[ tr.getPeptideRef() ] = tmp

    for key, value in trmap.iteritems():
        print key, value
        transition_group = getTransitionGroup(exp, targeted, key, value, chrom_map)
        picker.pickTransitionGroup(transition_group);
        for mrmfeature in transition_group.getFeatures():
            features = mrmfeature.getFeatures()
            for f in features:
                # TODO
                # f.getConvexHulls().clear()
                f.ensureUniqueId()

            mrmfeature.setSubordinates(features) # add all the subfeatures as subordinates
            output.push_back(mrmfeature)

    return output

def main(options):
    out = options.outfile
    chromat_in = options.infile
    traml_in = options.traml_in

    pp = pyopenms.MRMTransitionGroupPicker()

    pp_params = pp.getDefaults();
    pp_params.setValue("PeakPickerMRM:remove_overlapping_peaks", options.remove_overlapping_peaks, '')
    pp_params.setValue("PeakPickerMRM:method", options.method, '')
    pp.setParameters(pp_params);

    chromatograms = pyopenms.MSExperiment()
    fh = pyopenms.FileHandler()
    fh.loadExperiment(chromat_in, chromatograms)
    targeted = pyopenms.TargetedExperiment();
    tramlfile = pyopenms.TraMLFile();
    tramlfile.load(traml_in, targeted);

    output = algorithm(chromatograms, targeted, pp)

    pyopenms.FeatureXMLFile().store(out, output);

def handle_args():
    import argparse

    usage = ""
    usage += "\nMRMTransitionGroupPicker picks transition groups in measured chromatograms (mzML)"

    parser = argparse.ArgumentParser(description = usage )
    parser.add_argument('--in', dest="infile", help = 'An input file containing chromatograms')
    parser.add_argument("--tr", dest="traml_in", help="TraML input file containt the transitions")
    parser.add_argument("--out", dest="outfile", help="Output file with annotated chromatograms")
    parser.add_argument("--remove_overlapping_peaks", dest="remove_overlapping_peaks", default="false", help="true/false", metavar='0.1', type=str)
    parser.add_argument("--method", dest="method", default="legacy", help="legacy/corrected", metavar='0.1', type=str)

    args = parser.parse_args(sys.argv[1:])
    return args

if __name__ == '__main__':
    options = handle_args()
    main(options)




from Types cimport *
from libcpp cimport bool
from MetaInfoInterface cimport *
from Acquisition cimport *
from String cimport *

cdef extern from "<OpenMS/METADATA/AcquisitionInfo.h>" namespace "OpenMS":

    cdef cppclass AcquisitionInfo(MetaInfoInterface):
        # wrap-inherits:
        #    MetaInfoInterface
        
        AcquisitionInfo()    nogil except +
        AcquisitionInfo(AcquisitionInfo)    nogil except +

        bool operator==(AcquisitionInfo) nogil except +
        bool operator!=(AcquisitionInfo) nogil except +

        String getMethodOfCombination() nogil except +
        void setMethodOfCombination(String method) nogil except +

        Size size() nogil except + #wrap-doc:Number a Acquisition objects
        Acquisition& operator[](int) nogil except + # wrap-upper-limit:size()
        void push_back(Acquisition)  nogil except + #wrap-doc:Append a Acquisition object
        void resize(size_t n) nogil except +
    
        # libcpp.vector uneccessary - iteration is possible w/o, no assignment w/ or w/o



from Types cimport *
from DefaultParamHandler cimport *
from PeptideIdentification cimport *
from TextFile cimport *
from PeptideAndProteinQuant cimport *
from ProteinResolver cimport *

cdef extern from "<OpenMS/ANALYSIS/QUANTITATION/QuantitativeExperimentalDesign.h>" namespace "OpenMS":
    
    cdef cppclass QuantitativeExperimentalDesign(DefaultParamHandler) :
        # wrap-inherits:
        #  DefaultParamHandler
        QuantitativeExperimentalDesign() nogil except +
        QuantitativeExperimentalDesign(QuantitativeExperimentalDesign) nogil except + #wrap-ignore
        void applyDesign2Resolver(ProteinResolver & resolver, TextFile & file_, StringList & fileNames) nogil except +



from Types cimport *
from libcpp cimport bool
from FeatureGroupingAlgorithm cimport *
from KDTreeFeatureMaps cimport *
from FeatureDistance cimport *
from ProgressLogger cimport *

cdef extern from "<OpenMS/ANALYSIS/MAPMATCHING/FeatureGroupingAlgorithmKD.h>" namespace "OpenMS":
    
    cdef cppclass ClusterProxyKD "OpenMS::ClusterProxyKD":
        ClusterProxyKD() nogil except +
        ClusterProxyKD(ClusterProxyKD) nogil except +
        ClusterProxyKD(Size size, double avg_distance, Size center_index) nogil except +
        bool operator<(ClusterProxyKD & rhs) nogil except +
        bool operator!=(ClusterProxyKD & rhs) nogil except +
        bool operator==(ClusterProxyKD & rhs) nogil except +
        Size getSize() nogil except +
        bool isValid() nogil except +
        double getAvgDistance() nogil except +
        Size getCenterIndex() nogil except +




from Types cimport *
from DefaultParamHandler cimport *
from Matrix cimport *
from String cimport *
from Peak2D cimport *

cdef extern from "<OpenMS/ANALYSIS/QUANTITATION/IsobaricQuantitationMethod.h>" namespace "OpenMS::IsobaricQuantitationMethod":

    cdef cppclass IsobaricChannelInformation "OpenMS::IsobaricQuantitationMethod::IsobaricChannelInformation":
        IsobaricChannelInformation(IsobaricChannelInformation) nogil except + #wrap-ignore
        String name
        Int id
        String description
        double center
        Int channel_id_minus_2
        Int channel_id_minus_1
        Int channel_id_plus_1
        Int channel_id_plus_2
        IsobaricChannelInformation(String name, Int id_, String description, double center, Int channel_id_minus_2, Int channel_id_minus_1, Int channel_id_plus_1, Int channel_id_plus_2) nogil except +



from Types cimport *
from libcpp cimport bool
from libcpp.map cimport map as libcpp_map
from DefaultParamHandler cimport *
from MSSpectrum cimport *
from Peak1D cimport *

cdef extern from "<OpenMS/FILTERING/TRANSFORMERS/PeakMarker.h>" namespace "OpenMS":
    
    cdef cppclass PeakMarker(DefaultParamHandler) :
        # wrap-inherits:
        #  DefaultParamHandler
        PeakMarker() nogil except +
        PeakMarker(PeakMarker) nogil except +
        # see child classes
        # void apply(libcpp_map[ double, bool ] & , MSSpectrum & ) nogil except +
        String getProductName() nogil except +




from libcpp.vector cimport vector as libcpp_vector
from Types cimport *
from String cimport *
from TargetedExperimentHelper cimport *

cdef extern from "<OpenMS/ANALYSIS/MRM/ReactionMonitoringTransition.h>" namespace "OpenMS":

    cdef cppclass ReactionMonitoringTransition(CVTermList):
        # wrap-inherits:
        #    CVTermList

        ReactionMonitoringTransition()                  nogil except +
        ReactionMonitoringTransition(ReactionMonitoringTransition)   nogil except + #wrap-ignore
        String getName()                           nogil except +
        String getNativeID()                       nogil except +
        String getPeptideRef()                     nogil except +
        void setName(String name)                  nogil except +
        void setNativeID(String name)              nogil except +
        void setPeptideRef(String peptide_ref)     nogil except +

        double getProductMZ()         nogil except +
        void setProductMZ(double)     nogil except +

        double getPrecursorMZ()       nogil except +
        void setPrecursorMZ(double)   nogil except +
      
        DecoyTransitionType getDecoyTransitionType() nogil except +

        void setCompoundRef(const String & compound_ref)nogil except +
        String  getCompoundRef()nogil except +

        bool hasPrecursorCVTerms() nogil except +
        void setPrecursorCVTermList(CVTermList & list_)nogil except +
        void addPrecursorCVTerm(CVTerm & cv_term)nogil except +
        CVTermList getPrecursorCVTermList()nogil except +

        void addProductCVTerm(CVTerm & cv_term)nogil except +

        libcpp_vector[ TraMLProduct ]  getIntermediateProducts()nogil except +
        void addIntermediateProduct(TraMLProduct product)nogil except +
        void setIntermediateProducts(libcpp_vector[ TraMLProduct ] & products)nogil except +

        void setProduct(TraMLProduct product)nogil except +
        TraMLProduct getProduct()nogil except +
        void setRetentionTime(RetentionTime rt)nogil except +
        RetentionTime getRetentionTime()nogil except +

        void setPrediction(Prediction & prediction)nogil except +
        void addPredictionTerm(CVTerm & prediction)nogil except +
        bool hasPrediction() nogil except +
        Prediction getPrediction()nogil except +

        void setDecoyTransitionType(DecoyTransitionType & d)nogil except +

        double getLibraryIntensity()nogil except +
        void setLibraryIntensity(double intensity)nogil except +

        int getProductChargeState() nogil except +
        bool isProductChargeStateSet() nogil except +

        bool isDetectingTransition() nogil except +
        void setDetectingTransition(bool val) nogil except +

        bool isIdentifyingTransition() nogil except +
        void setIdentifyingTransition(bool val) nogil except +

        bool isQuantifyingTransition() nogil except +
        void setQuantifyingTransition(bool val) nogil except +

cdef extern from "<OpenMS/ANALYSIS/MRM/ReactionMonitoringTransition.h>" namespace "OpenMS::ReactionMonitoringTransition":

    cdef enum DecoyTransitionType:

        UNKNOWN, TARGET, DECOY




from ConsensusMap cimport *
from String cimport *
from PeakFileOptions cimport *

cdef extern from "<OpenMS/FORMAT/ConsensusXMLFile.h>" namespace "OpenMS":

    cdef cppclass ConsensusXMLFile:
        ConsensusXMLFile() nogil except +

        void load(String, ConsensusMap &) nogil except+
        void store(String, ConsensusMap &) nogil except+

        PeakFileOptions getOptions() nogil except +



# part of the SpectraFilters
from MSSpectrum cimport *
from MSExperiment cimport *
from ChromatogramPeak cimport *
from Peak1D cimport *
from String cimport *
from DefaultParamHandler cimport *

cdef extern from "<OpenMS/FILTERING/TRANSFORMERS/SpectraMerger.h>" namespace "OpenMS":

    cdef cppclass SpectraMerger(DefaultParamHandler):
        # wrap-inherits:
        #    DefaultParamHandler

        SpectraMerger()            nogil except +
        SpectraMerger(SpectraMerger) nogil except + #wrap-ignore

        void mergeSpectraBlockWise(MSExperiment & exp) nogil except +
        void mergeSpectraPrecursors(MSExperiment & exp) nogil except +
        void average(MSExperiment & exp, String average_type) nogil except +




from ProgressLogger cimport *
from DefaultParamHandler cimport *
from MSExperiment cimport *
from MSSpectrum cimport *
from Peak1D cimport *
from ChromatogramPeak cimport *
from MSChromatogram cimport *

cdef extern from "<OpenMS/FILTERING/NOISEESTIMATION/SignalToNoiseEstimatorMedian.h>" namespace "OpenMS":

    cdef cppclass SignalToNoiseEstimatorMedian[SpectrumT]:
        # wrap-instances:
        #   SignalToNoiseEstimatorMedian := SignalToNoiseEstimatorMedian[MSSpectrum]

        SignalToNoiseEstimatorMedian() nogil except +
        SignalToNoiseEstimatorMedian(SignalToNoiseEstimatorMedian) nogil except +

        void init(MSSpectrum & spectrum) nogil except +
        double getSignalToNoise(Size index) nogil except +

        double getSparseWindowPercent() nogil except +
        double getHistogramRightmostPercent() nogil except +

        # Functions for SignalToNoiseEstimatorMedianChrom[MSChromatogram]
        # use wrap-ignore because autowrap cannot handle them at the moment
        # see addons/SignalToNoiseEstimatorMedianChrom.pyx for the implementation
        void init(MSChromatogram & spectrum) nogil except + #wrap-ignore

cdef extern from "<OpenMS/FILTERING/NOISEESTIMATION/SignalToNoiseEstimatorMedian.h>" namespace "OpenMS::SignalToNoiseEstimatorMedian":

    cdef enum IntensityThresholdCalculation "OpenMS::SignalToNoiseEstimatorMedianChrom::IntensityThresholdCalculation":
        # wrap-attach:
        #     SignalToNoiseEstimatorMedian
        MANUAL
        AUTOMAXBYSTDEV
        AUTOMAXBYPERCENT



from MSSpectrum cimport *
from MSExperiment cimport *
from ChromatogramPeak cimport *
from Peak1D cimport *
from Param cimport *
from DefaultParamHandler cimport *
from ProgressLogger cimport *

cdef extern from "<OpenMS/TRANSFORMATIONS/RAW2PEAK/PeakPickerCWT.h>" namespace "OpenMS":

    cdef cppclass PeakPickerCWT(DefaultParamHandler, ProgressLogger):
        # wrap-inherits:
        #    DefaultParamHandler
        #    ProgressLogger

        PeakPickerCWT()                  nogil except +
        PeakPickerCWT(PeakPickerCWT)   nogil except + #wrap-ignore

        void pick(MSSpectrum & input, MSSpectrum & output) nogil except +
        void pickExperiment(MSExperiment & input, MSExperiment & output) nogil except +
        double estimatePeakWidth(MSExperiment & input) nogil except +




from Types cimport *
from libcpp.vector cimport vector as libcpp_vector
from libcpp.string cimport string as libcpp_string

cdef extern from "<OpenMS/OPENSWATHALGO/DATAACCESS/TransitionExperiment.h>" namespace "OpenSwath":

    cdef cppclass LightTransition:
        LightTransition() nogil except +
        LightTransition(LightTransition) nogil except +
        libcpp_string transition_name
        libcpp_string peptide_ref
        double library_intensity
        double product_mz
        double precursor_mz

        int fragment_charge
        bool decoy 
        bool detecting_transition
        bool quantifying_transition
        bool identifying_transition

        int getProductChargeState() nogil except +
        bool isProductChargeStateSet() nogil except +
        libcpp_string getNativeID() nogil except +
        libcpp_string getPeptideRef() nogil except +
        double getLibraryIntensity() nogil except +
        void setLibraryIntensity(double l) nogil except +
        double getProductMZ() nogil except +
        double getPrecursorMZ() nogil except +

        libcpp_string getCompoundRef() nogil except +

        # Detecting / quantifying / identifying transitions
        void setDetectingTransition (bool d) nogil except +
        bool isDetectingTransition() nogil except +
        void setQuantifyingTransition (bool q) nogil except +
        bool isQuantifyingTransition() nogil except +
        void setIdentifyingTransition (bool i) nogil except +
        bool isIdentifyingTransition() nogil except +

    cdef cppclass LightModification:
        LightModification() nogil except +
        LightModification(LightModification) nogil except +

        int location
        int unimod_id

    cdef cppclass LightCompound:
        LightCompound() nogil except +
        LightCompound(LightCompound) nogil except +

        double rt
        double drift_time
        int charge
        libcpp_string sequence
        libcpp_vector[libcpp_string] protein_refs
        libcpp_string peptide_group_label
        libcpp_string id
        libcpp_string sum_formula
        libcpp_string compound_name

        libcpp_vector[LightModification] modifications

        void setDriftTime(double d) nogil except +
        double getDriftTime() nogil except +

        int getChargeState() nogil except +
        bool isPeptide() nogil except +
        void setChargeState(int ch) nogil except +

    cdef cppclass LightProtein:
        LightProtein() nogil except +
        LightProtein(LightProtein) nogil except +
        libcpp_string id
        libcpp_string sequence

    cdef cppclass LightTargetedExperiment:

        LightTargetedExperiment() nogil except +
        LightTargetedExperiment(LightTargetedExperiment &) nogil except +

        libcpp_vector[LightTransition] transitions
        libcpp_vector[LightCompound] compounds
        libcpp_vector[LightProtein] proteins
        libcpp_vector[LightTransition] getTransitions()  nogil except +

        libcpp_vector[LightCompound] getCompounds() nogil except +
        libcpp_vector[LightProtein] getProteins() nogil except +

        LightCompound getCompoundByRef(libcpp_string & ref) nogil except +
        LightCompound getPeptideByRef(libcpp_string & ref) nogil except +

